function [final_table,times]  = construct_final_table(suitable_times_ACC,...
    suitable_times_LC, ...
    AUDI, t_start, t_end)

    % Constants
    print_figures       = false;
    
    % Join tables into final table
    final_table 	= timetable(AUDI.Time, ...
        ones(numel(suitable_times_ACC.Time),1), ...
        ones(numel(suitable_times_LC.Time),1), ...
        'VariableNames',{'suitable_times_ACC','suitable_times_LC'});
    final_table  	= outerjoin(AUDI,final_table);
    
    % Find sampling intervals
    dt              = diff(final_table.Time);
    dt(dt > seconds(2)) = 0;
    times.initial   = sum(dt);

    % Combine all partial results into final table
    for i = 1 : size(suitable_times_ACC,2)
        final_table.suitable_times_ACC  = ...
            final_table.suitable_times_ACC .* suitable_times_ACC{:,i};
    end
    for i = 1 : size(suitable_times_LC,2)
        final_table.suitable_times_LC	= ...
            final_table.suitable_times_LC .* suitable_times_LC{:,i};
    end
    
    % If requested, print figures ...
    if print_figures
        sFont = 14;
        hFig = figure();
        set(hFig, 'Position', [100 100 1400 800])
        set(gcf,'Color','white');

        sp1 = subplot(6,1,1:5);
        hold on; grid minor

        plot(final_table.Time,final_table.speed_kal, 'LineWidth',1.5)
        title('Vehicle speed'); 
        xlabel('Time','FontSize',sFont,'FontWeight','normal');     
        ylabel('Speed, m/s','FontSize',sFont,'FontWeight','normal');
        legend('Speed')
        set(gca,'FontSize',sFont)

        sp2 = subplot(6,1,6);
        hold on; grid minor

        plot(final_table.Time(final_table.suitable_times_ACC == 0), ...
            1-final_table.suitable_times_ACC(final_table.suitable_times_ACC == 0), ...
            '*','LineWidth',2.5,'MarkerSize',10)
        plot(final_table.Time(final_table.suitable_times_LC == 0), ...
            1-final_table.suitable_times_LC(final_table.suitable_times_LC == 0), ...
            '*','LineWidth',2.5,'MarkerSize',10)

        title(' '); 
        xlabel('Time','FontSize',sFont,'FontWeight','normal');     
        ylabel(' ','FontSize',sFont,'FontWeight','normal');
        legend('ACC','LC', ...
            'Orientation','Horizontal','Location','North')
        set(gca,'FontSize',sFont)

        linkaxes([sp1,sp2],'x')
        set(zoom(figure(hFig)),'Motion','horizontal')
        set(pan(figure(hFig)),'Motion','horizontal')
    end
    
    % Print suitable times for ACC system activation
    times.ACC       = datestr(seconds(sum(abs(final_table.suitable_times_ACC - 1))*0.1),'HH:MM:SS');
    
    % Print suitable times for lane-changes
    tmp_suitable_times_LC   = final_table.suitable_times_LC;
    tmp_suitable_times_LC(~final_table.suitable_times_ACC) = 1;
    times.LC        = datestr(seconds(sum(abs(tmp_suitable_times_LC - 1)*0.1)),'HH:MM:SS');
    
    % Only keep values for which both ACC and LC flags indicate true
    final_table = final_table(final_table.suitable_times_ACC == 1 & ...
        final_table.suitable_times_LC == 1,:);
    
    % Remove ACC and LC flag columns from table
    cols        = find(ismember(fieldnames(final_table),{'suitable_times_ACC',...
        'suitable_times_LC'}))';
    final_table(:,cols)   	= [];

    % Output requested data
    final_table         = final_table(final_table.Time >= t_start & ...
        final_table.Time <= t_end,:);
    final_table.Properties.VariableNames    = {'longitude','latitude',...
        'speed_lim','traffic_light','invalid',...
        'x','v','a','ID','gear','s','s_dot','vl','regime',...
        'regime_enum'};
    
    % Also print final remaining time
    dt              = diff(final_table.Time);
    dt(dt > seconds(2)) = 0;
    times.final     = sum(dt);
end





