function [AUDI]     = detect_driving_regimes(dt, GPS_kalman, CAN, ME, ...
                            s_c, v_c, a_c, a_c_kin, T_c)

    % Constants
    print_figures       = false;
    N_smooth            = 25;
    
    % First row to final table
    AUDI                = GPS_kalman(:,ismember(fieldnames(GPS_kalman),...
        {'longitude','latitude','dist_kal','speed_kal',...
        'acc_kal','v_lim','traffic_lights','invalid'}));
    AUDI.ID             = ME.ID;
    AUDI.gear           = CAN.gearstick_position;
    
    %   V1.0 - Add one-second reference correction (see script
    %       "mobileye_investigation_v1.m")
    %   V2.0 - Removed compensation due to new insights
    tmp_s               = ME.x;
    tmp_s(tmp_s == 0)   = Inf;

    % Calculate velocity of lead-vehicle
    tmp_vl              = (AUDI.speed_kal + ME.vx);
    
    % Calcultate change in distance-gap
    tmp_s_dot           = ME.vx;

    % And acceleration of ego-vehicle
    tmp_a               = AUDI.acc_kal;

    % Write distance-gap and lead vehicle velocity to final table
    AUDI.s              = tmp_s;
    AUDI.s_dot          = tmp_s_dot;
    AUDI.vl             = tmp_vl;
        
    %% Regime detection
    % For retime detection, do not use raw data, but derive dependent
    % variables from selected data.
    %   -> Use velocity to compute driven distance and acceleration
    %       ego-vehicle
    %   -> Use gap to compute velocity lead-vehicle
    [AUDI_regimes]      = compute_dependent_vars_regimes(AUDI, dt);
    
    AUDI_regimes.x_ddot = movmean(AUDI_regimes.x_ddot, N_smooth, 'omitnan');
    
    tmp_a               = AUDI_regimes.x_ddot;
    tmp_x_dot           = AUDI_regimes.x_dot;
    tmp_s               = AUDI_regimes.s;
    tmp_s_dot           = AUDI.s_dot(ismember(AUDI.Time,AUDI_regimes.Time));
    tmp_vl              = movmean(tmp_x_dot + tmp_s_dot, N_smooth, 'omitnan'); 
    AUDI_regimes.xl_dot = tmp_vl;
    tmp_s(tmp_s == Inf) = NaN;
    tmp_a_kin           = (tmp_x_dot - tmp_vl) .^2 ./ (2 .* tmp_s);
    AUDI_regimes.a_kin  = tmp_a_kin;
    tmp_s(isnan(tmp_s)) = Inf;
    tmp_T               = tmp_s ./ tmp_x_dot;
    
    % Initialize regime table
    tmp_regime        	= cell(size(AUDI_regimes,1),1);
    tmp_regime_enum  	= NaN(size(tmp_regime));
    AUDI.regime         = cell(size(AUDI,1),1);
    AUDI.regime_enum    = NaN(size(AUDI.regime));
    
    progressbar(0)
    for i = 1 : size(AUDI_regimes,1)
        if rem(i,10000) == 0
            progressbar((i-1)/size(AUDI_regimes,1))
        end
        
        % Cruising
        %   - Speed above limit v_c
        %   - Gap (s/v) above limit T_c
        if tmp_x_dot(i) >= v_c && ...
                tmp_T(i) >= T_c && ...
                abs(tmp_a(i)) <= a_c
            
            tmp_regime(i)       = {'cruising'};
            tmp_regime_enum(i)	= 1;
            
        elseif tmp_T(i) >= T_c && ...
                tmp_a(i) >= a_c

            tmp_regime(i)       = {'free_acceleration'};
            tmp_regime_enum(i)	= 2;

        % Approaching
        %   - Ego velocity higher than lead-vehicle velocity (v > v_l)
        %   - Gap (s/v) above limit T_c
        %   - Kinematic deceleration above a_c (v-v_l)^2 / (2s) > a_c
        elseif tmp_x_dot(i) >= tmp_vl(i) && ...
                tmp_T(i) >= T_c && ...
                tmp_a_kin(i) >= a_c_kin

            tmp_regime(i)      = {'approaching'};
            tmp_regime_enum(i) = 3;

        % Standing
        %   - Gap below limit s_c
        elseif tmp_s(i) <= s_c

            tmp_regime(i)      = {'standing'};
            tmp_regime_enum(i) = 4;
            
        % Following
        %   - Gap (s/v) below limit T_c
        %   - None of above conditions applies
        elseif tmp_T(i) <= T_c

            tmp_regime(i)      = {'following'};
            tmp_regime_enum(i) = 5;

        % Inconsistent
        %   - Otherwise
        else
            tmp_regime(i)      = {'inconsistent'};
            tmp_regime_enum(i) = 6;
        end
    end
    progressbar(1)
    
    AUDI.regime(ismember(AUDI.Time,AUDI_regimes.Time)) = ...
        tmp_regime;
    AUDI.regime_enum(ismember(AUDI.Time,AUDI_regimes.Time)) = ...
        tmp_regime_enum;
    
    fprintf(char(strcat(...
        {'Regions \n\n%.1f%% cruising\n%.1f%% free acceleration\n%.1f%% approaching\n'}, ...
        {'%.1f%% standing\n%.1f%% following\n%.1f%% inconsistent\n'})), ...
        sum(strcmpi(tmp_regime, 'cruising')) ./ numel(tmp_regime) * 100, ...
        sum(strcmpi(tmp_regime, 'free_acceleration')) ./ numel(tmp_regime) * 100, ...
        sum(strcmpi(tmp_regime, 'approaching')) ./ numel(tmp_regime) * 100, ...
        sum(strcmpi(tmp_regime, 'standing')) ./ numel(tmp_regime) * 100, ...
        sum(strcmpi(tmp_regime, 'following')) ./ numel(tmp_regime) * 100, ...
        sum(strcmpi(tmp_regime, 'inconsistent')) ./ numel(tmp_regime) * 100)

    %% If requested: print figures
    
    if print_figures
%         times_vec   = [datetime([2018 06 13 14 40 00]), datetime([2018 06 13 14 49 00])];
        times_vec   = [datetime([2018 06 13 07 01 00]), datetime([2018 06 13 07 02 00]); ...
            datetime([2018 06 13 07 05 00]), datetime([2018 06 13 07 06 00]); ...
            datetime([2018 06 13 07 23 00]), datetime([2018 06 13 07 24 00]); ...
            datetime([2018 06 13 10 28 00]), datetime([2018 06 13 10 29 00]); ...
            datetime([2018 06 13 10 12 45]), datetime([2018 06 13 10 14 45])];
            
        titles_vec  = ...
            [{'Approaching a slower vehicle'}; ...
            {'Accelerating while car-following'}; ...
            {'Decelerating while car-following'}; ...
            {'Hard braking while car-following'}; ...
            {'Leaving faster vehicle'}];            
    
        for i = 1 : size(times_vec,1)
            sFont = 14;
            hFig3 = figure();
            set(hFig3, 'Position', [100 100 1400 800])
            set(gcf,'Color','white');

            sp1 = subplot(3,1,1);
            hold on; grid minor
            plot(AUDI.Time,10.*(AUDI.s ./ AUDI.speed_kal),'LineWidth',1.5)
            plot(ME.Time,ones(size(ME.Time)).*T_c.*10,'LineWidth',1.5)
            plot(AUDI.Time,AUDI.s,'LineWidth',1.5)
            plot(ME.Time,ME.ID,'LineWidth',1.5)
            plot(AUDI.Time,(AUDI.regime_enum-1) .* 25,'LineWidth',1.5)

            title(titles_vec{i}); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel(' ','FontSize',sFont,'FontWeight','normal');
            legend('10 * time-gap, s','10 * T_c','Distance-gap, m',...
                'Obstacle ID','Regime')
            set(gca,'FontSize',sFont)

            sp2 = subplot(3,1,2);
            hold on; grid minor
            plot(AUDI_regimes.Time,AUDI_regimes.x_dot,'LineWidth',1.5)
            tmp_vl(tmp_vl == 0) = NaN;
            plot(AUDI_regimes.Time,AUDI_regimes.xl_dot,'LineWidth',1.5)

            % title(' '); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel('Velocity, m/s','FontSize',sFont,'FontWeight','normal');
            legend('Speed (ego), m/s','Speed (lead), m/s')
            set(gca,'FontSize',sFont)

            sp3 = subplot(3,1,3);
            hold on; grid minor
            plot(AUDI_regimes.Time,AUDI_regimes.x_ddot,'LineWidth',1.5)
            plot(ME.Time,ones(size(ME.Time)).*a_c,'LineWidth',1.5)
            plot(ME.Time,-ones(size(ME.Time)).*a_c,'LineWidth',1.5)
            plot(AUDI_regimes.Time,AUDI_regimes.a_kin,'LineWidth',1.5)

            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel(' ','FontSize',sFont,'FontWeight','normal');
            legend('a','a_c','-a_c','Kinematic deceleration')
            set(gca,'FontSize',sFont)

            linkaxes([sp1,sp2,sp3],'x')
            set(gca,'XLim',times_vec(i,:))
            
            % Plot position-time graph
            sFont = 14;
            hFig4 = figure();
            set(hFig4, 'Position', [100 100 1400 800])
            set(gcf,'Color','white');

            cum_dist    = cumtrapz(.1,AUDI.speed_kal);
            dist_lead   = cum_dist + AUDI.s;
            dist_lead(dist_lead == Inf) = NaN;

            sp1 = subplot(3,1,1:2);
            hold on; grid minor
            plot(AUDI.Time,cum_dist,'LineWidth',1.5)
            plot(AUDI.Time,dist_lead,'LineWidth',1.5)

            title(titles_vec{i}); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel(' ','FontSize',sFont,'FontWeight','normal');
            legend('Ego-vehicle','Lead-vehicle')
            set(gca,'FontSize',sFont)

            sp2 = subplot(3,1,3);
            hold on; grid minor
            plot(AUDI.Time,AUDI.ID,'LineWidth',1.5)

            title(titles_vec{i}); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel(' ','FontSize',sFont,'FontWeight','normal');
            legend('ID')
            set(gca,'FontSize',sFont)

            linkaxes([sp1, sp2],'x')
            set(gca,'XLim',times_vec(i,:))
        end
    end
end