function [suitable_times_ACC]   = find_suitable_times_ACC(CAN,effective_duration)
    global sFont figWidth figHeight figWidth_panel figHeight_panel cmap

    % Constants
    print_figures       = true;

    % Find times at which ACC state is indicated as "ON"
    i_ACC_state_ON      = find(diff(CAN.acc_state) == 1);
    i_ACC_state_OFF     = find(diff(CAN.acc_state) == -1);
    t_ACC_state         = [CAN.Time(i_ACC_state_ON), CAN.Time(i_ACC_state_OFF)];

    % Find times at which ACC overruling is indicated as "TRUE"
    i_ACC_overrule_ON	= find(diff(CAN.acc_overrule) == 1);
    i_ACC_overrule_OFF	= find(diff(CAN.acc_overrule) == -1);
    t_ACC_overrule      = [CAN.Time(i_ACC_overrule_ON), CAN.Time(i_ACC_overrule_OFF)];

    % Find times at which manual throttle signal is logged
    throttle_tmp        = CAN(:,ismember(fieldnames(CAN),{'throttle_position'}));
    % Manipulate first and last value to obtain valid results
    throttle_tmp.throttle_position(end) = 0;
    throttle_tmp.throttle_position(1) = 0;
    throttle_tmp.throttle_position(throttle_tmp.throttle_position > 0) = 1;
    i_throttle_ON       = find(diff(throttle_tmp.throttle_position) == 1);
    i_throttle_OFF      = find(diff(throttle_tmp.throttle_position) == -1);
    t_throttle          = [CAN.Time(i_throttle_ON), CAN.Time(i_throttle_OFF)];

    % Find times at which manual brake pedal signal is logged
    brake_tmp           = CAN(:,ismember(fieldnames(CAN),{'brake_position'}));
    % Manipulate first and last value to obtain valid results
    brake_tmp.brake_position(end) = 0;
    brake_tmp.brake_position(1) = 0;
    brake_tmp.brake_position(brake_tmp.brake_position > 0) = 1;
    i_brake_ON          = find(diff(brake_tmp.brake_position) == 1);
    i_brake_OFF         = find(diff(brake_tmp.brake_position) == -1);
    t_brake             = [CAN.Time(i_brake_ON), CAN.Time(i_brake_OFF)];

    % Calculate interval within which ACC state and overrule effects are visible
    ACC                 = timetable(CAN.Time, ...
        zeros(numel(CAN.Time),1), ...
        ones(numel(CAN.Time),1), ...
        ones(numel(CAN.Time),1), ...
        ones(numel(CAN.Time),1), ...
        'VariableNames',{'state','overrule',...
        'throttle','brake'});

    % Find start and end indices of all signals
    for i = 1 : size(t_ACC_state,1)
        [~,i_start]	= min(abs(ACC.Time - (t_ACC_state(i,1) + seconds(effective_duration(2)))));
        [~,i_end] 	= min(abs(ACC.Time - (t_ACC_state(i,2) + seconds(effective_duration(1)))));
        ACC.state(i_start:i_end)        = 1;
    end
    for i = 1 : size(t_ACC_overrule,1)
        [~,i_start]	= min(abs(ACC.Time - (t_ACC_overrule(i,1) + seconds(effective_duration(1)))));
        [~,i_end] 	= min(abs(ACC.Time - (t_ACC_overrule(i,2) + seconds(effective_duration(2)))));
        ACC.overrule(i_start:i_end)     = 0;
    end
    for i = 1 : size(t_throttle,1)
        [~,i_start]	= min(abs(ACC.Time - (t_throttle(i,1) + seconds(effective_duration(1)))));
        [~,i_end] 	= min(abs(ACC.Time - (t_throttle(i,2) + seconds(effective_duration(2)))));
        ACC.throttle(i_start:i_end)     = 0;
    end
    for i = 1 : size(t_brake,1)
        [~,i_start]	= min(abs(ACC.Time - (t_brake(i,1) + seconds(effective_duration(1)))));
        [~,i_end] 	= min(abs(ACC.Time - (t_brake(i,2) + seconds(effective_duration(2)))));
        ACC.brake(i_start:i_end)     = 0;
    end
    
    %% Print figure
    if print_figures
        hFig = figure();
        set(hFig, 'Position', [100 100 figWidth .7*figHeight])
        set(gcf,'Color','white');

        % Add plots to figure
        hold on; grid minor
        plot(CAN.Time,CAN.vehicle_speed ./ max(CAN.vehicle_speed), 'LineWidth',1.5)
        plot(CAN.Time,CAN.throttle_position ./ max(CAN.throttle_position), 'LineWidth',1.5)
        plot(CAN.Time,CAN.brake_position ./ max(CAN.brake_position), 'LineWidth',1.5)
        plot(CAN.Time,(ACC.state) .* 1.00 - 0.1, '*', 'LineWidth',1.5)
        plot(CAN.Time,(1-ACC.overrule) .* 0.95 - 0.1, '*', 'LineWidth',1.5)
        plot(CAN.Time,(1-ACC.throttle) .* 0.90 - 0.1, '*', 'LineWidth',1.5)
        plot(CAN.Time,(1-ACC.brake) .* 0.85 - 0.1, '*', 'LineWidth',1.5)
        
        % Set figure properties
        ylabel('Normalised value, -','FontSize',sFont,'FontWeight','normal');
        legend('Velocity','Manual Throttle Input','Manual Brake Input',...
            'ACC State Flag',...
            'ACC Overrule Flag',...
            'Manual Throttle Flag', ...
            'Manual Brake Flag')
        set(gca,'FontSize',sFont)
        set(gca,'YLim',[0 1])

        set(gca,'XLim',[datetime([2018 06 13 14 27 00]), datetime([2018 06 13 14 29 00])])
        datetick('x','HH:MM:ss','keeplimits','keepticks')

        set(zoom(figure(hFig)),'Motion','horizontal')
        set(pan(figure(hFig)),'Motion','horizontal')
    end
    
    suitable_times_ACC  = ACC;
end






