function [data_out]     = select_ACL(data,t_start,t_end,Fs)

    % OUTPUT
    %   ACL .x      [m/s^2]
    %       .y      [m/s^2]
    %       .z      [m/s^2]
    
    % Optional
    %   Print figure showing linear trend and bias in the accelerometer
    %   signal
    print_figure = false;
    
    % Constants
    t_thresh    = 2;
    
    % Get values and remove some inconsistent values
    ACL         = data.DL004.Advantech_IVCP.T_ivcp_raw;
    ACL         = ACL(:,ismember(fieldnames(ACL),{'acceleration_x','acceleration_y','acceleration_z'}));
    ACL.Properties.VariableContinuity  = {'continuous','continuous','continuous'};
    ACL.acceleration_x(abs(ACL.acceleration_x)>50) = NaN;
    ACL.acceleration_x(abs(ACL.acceleration_y)>50) = NaN;
    ACL.acceleration_x(abs(ACL.acceleration_z)>50) = NaN;
    ACL         = ACL(ACL.Time >= t_start,:);
    ACL         = ACL(ACL.Time <= t_end,:);
    
    % Remove duplicate samping times
    [n, bin]    = histc(datenum(ACL.Time), unique(datenum(ACL.Time)));
    multiple    = find(n > 1);
    index       = find(ismember(bin, multiple));
    rm_index    = index(diff(index)==1);
    ACL(rm_index,:)  = [];
  
    ACL.Properties.VariableNames = {'x','y','z'};

    % Find missing data
    tmp_t_missing   = (ACL.Time(2:end) - ACL.Time(1:end-1)) >= seconds(t_thresh);
    tmp_idx_missing = [0;find(tmp_t_missing);size(ACL,1)];
    
    % For each of the non-missing intervals, retime the signal and
    % write to the final table
    ACL_out         = timetable();
    for i = 1 : numel(tmp_idx_missing) - 1
        TT      = ACL(tmp_idx_missing(i)+1 : tmp_idx_missing(i+1),:);
        TT		= retime(TT,'Regular','SampleRate',10);
        ACL_out = [ACL_out; TT];
    end
    ACL             = ACL_out;
    
    clear CAN_out TT tmp*
    
    % Load velocity signal
    CAN         = data.DL004.Advantech_VCIL_AUDI_A4.T_can_vehicle_speed_raw;
    CAN         = CAN(:,ismember(fieldnames(CAN),{'vehicle_speed'}));
    CAN.Properties.VariableContinuity 	= {'continuous'};
    
    % Set velocity around missing points to 0 to avoid errors in
    % interpolation
    tmp_t_missing   = (CAN.Time(2:end) - CAN.Time(1:end-1)) >= seconds(t_thresh);
    tmp_idx_missing = find(tmp_t_missing);
    CAN.vehicle_speed(tmp_idx_missing) = 0;
    CAN.vehicle_speed(tmp_idx_missing + 1) = 0;
    CAN        	= retime(CAN,'Regular','SampleRate',Fs);
    
    % Only keep common times
    [~,ia,ib]   = intersect(ACL.Time,CAN.Time);
    ACL         = ACL(ia,:);
    CAN         = CAN(ib,:);
    
    % Set the acceleration signal to zero at times the velocity is zero
    tmp_speed       = CAN.vehicle_speed;
    tmp_speed(1)    = 0;
    tmp_speed(end)  = 0;
    tmp_speed(tmp_speed ~= 0) = 1;
    ACL.x           = ACL.x .* tmp_speed;
    
    % Correct all accelerations by the gravitational constant g
    g       = 9.80665;
    ACL.x   = ACL.x .* g;
    ACL.y   = ACL.y .* g;
    ACL.z   = ACL.z .* g;
    
    % Detrend the acceleration signal between these times
    tmp_t_start = find(diff(tmp_speed) == 1);
    tmp_t_end   = find(diff(tmp_speed) == -1);
    
    ACL_mean    = NaN(size(ACL.x));
    ACL_trend   = NaN(size(ACL.x));
    for i = 1 : numel(tmp_t_start)
        idxs = tmp_t_start(i) + 1 : tmp_t_end(i);
        if print_figure
            ACL_mean(idxs)     = ...
                ones(numel(idxs),1) .* ...
                mean(ACL.x(idxs));
            ACL_trend(idxs)    = ...
                ACL.x(idxs) - ...
                detrend(ACL.x(idxs));
        end
        ACL.x(idxs) = ...
            ACL.x(idxs) - ...
            mean(ACL.x(idxs));
    end
    
    % Is print_figure is requested, do so...
    if print_figure
        sFont = 14;
        hFig = figure();
        set(hFig, 'Position', [100 100 1400 800])
        set(gcf,'Color','white');
        
        % Add plots to first subplot
        sp1 = subplot(2,1,1);
        hold on; grid minor
        plot(ACL_out.Time,ACL_out.x .* g,'LineWidth',1)
        plot(ACL.Time,ACL_mean,'LineWidth',1.5)
        plot(ACL.Time,ACL_trend,'LineWidth',1.5)
        
        % Add subplot properties
        title('Vehicle accelerations'); 
        xlabel('Time','FontSize',sFont,'FontWeight','normal');     
        ylabel('Acceleration, m/s^2','FontSize',sFont,'FontWeight','normal');
        legend('a','a_{mean}','a_{trend}',...
            'Orientation','Horizontal','Location','North')
        set(gca,'FontSize',sFont)
        
        % Add plots to second subplot
        sp2 = subplot(2,1,2);
        hold on; grid minor
        plot(CAN.Time,CAN.vehicle_speed,'LineWidth',1.5)
        
        % Add subplot properties
        title('Vehicle velocity'); 
        xlabel('Time','FontSize',sFont,'FontWeight','normal');     
        ylabel('Speed, m/s','FontSize',sFont,'FontWeight','normal');
        set(gca,'FontSize',sFont)
        
        linkaxes([sp1,sp2],'x')
        xlim([datetime([2018 06 13 09 52 48]),...
            datetime([2018 06 13 09 53 02])])
    end
    
    % Output data
    data_out    = ACL;
    
end












