function [data_out]     = select_CAN(data,t_start,t_end,Fs)

    % List of selected CAN-bus signals
    %   1 - vehicle_speed
    %   2 - acc_state
    %   3 - acc_overrule
    %   4 - throttle_position
    %   5 - brake_position
    %   6 - brake_light_status
    %   7 - indicator_right_status
    %   8 - indicator_left_status
    %   9 - gearstick_position

    % Constants
    t_thresh    = 2;
    
    CAN         = timetable();
    
    % Load vehicle_speed data
    TT         = data.DL004.Advantech_VCIL_AUDI_A4.T_can_vehicle_speed_raw;
    TT         = TT(:,ismember(fieldnames(TT),{'vehicle_speed'}));
    TT.Properties.VariableContinuity 	= {'continuous'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh);
    
    % Load acc_state data
    TT        	= data.DL004.Advantech_VCIL_AUDI_A4.T_can_acc_status_raw;
    TT      	= TT(:,ismember(fieldnames(TT),{'acc_state'}));
    TT.acc_state(TT.acc_state==2)       = 1;
    TT.Properties.VariableContinuity    = {'step'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh);
    CAN.acc_state(isnan(CAN.acc_state)) = 0;

    % Load acc_overrule data
    TT          = data.DL004.Advantech_VCIL_AUDI_A4.T_can_acc_overrule_raw;
    TT          = TT(:,ismember(fieldnames(TT),{'acc_overrule'}));
    TT.Properties.VariableContinuity    = {'step'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh);

    % Load throttle_position data
    TT      	= data.DL004.Advantech_VCIL_AUDI_A4.T_can_throttle_position_raw;
    TT        	= TT(:,ismember(fieldnames(TT),{'throttle_position'}));
    TT.Properties.VariableContinuity    = {'continuous'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh);
    
    % Load brake_position data
    TT          = data.DL004.Advantech_VCIL_AUDI_A4.T_can_brake_position_raw;
    TT          = TT(:,ismember(fieldnames(TT),{'brake_position'}));
    TT.Properties.VariableContinuity    = {'continuous'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh); 
    
    % Load brake_light data
    TT          = data.DL004.Advantech_VCIL_AUDI_A4.T_can_brake_light_raw;
    TT          = TT(:,ismember(fieldnames(TT),{'brake_light_status'}));
    TT.Properties.VariableContinuity    = {'step'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh);
    
    % Load indicator data
    TT          = data.DL004.Advantech_VCIL_AUDI_A4.T_can_indicator_raw;
    TT          = TT(:,ismember(fieldnames(TT),...
        {'indicator_right_status', 'indicator_left_status'}));
    TT.Properties.VariableContinuity    = {'step','step'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh);
    
    % Load gearstick position data
    TT          = data.DL004.Advantech_VCIL_AUDI_A4.T_can_gearstick_position_raw;
    TT          = TT(:,ismember(fieldnames(TT),{'gearstick_position'}));
    TT.Properties.VariableContinuity    = {'step'};
    CAN         = add_to_timetable(CAN,TT,Fs,t_thresh);
      
    % Edit some signals..
    CAN.brake_position(CAN.brake_position<0.1)                      = 0;
    CAN.throttle_position(CAN.throttle_position < 0.5)              = 0;
    CAN.indicator_left_status(isnan(CAN.indicator_left_status))     = 0;
    CAN.indicator_right_status(isnan(CAN.indicator_right_status))   = 0;
    CAN.acc_state(isnan(CAN.acc_state))                             = 0;
    CAN.acc_overrule(isnan(CAN.acc_overrule))                       = 0;
    CAN.brake_light_status(isnan(CAN.brake_light_status))           = 0;
    CAN.throttle_position   = CAN.throttle_position ./ ...
        max(CAN.throttle_position);
    CAN.brake_position      = CAN.brake_position ./ ...
        max(CAN.brake_position);
    
    CAN                 = CAN(CAN.vehicle_speed > 0,:);
    
    % Rewrite gearstick signal to human-readable string values
    GS_translate_num    = [0 1 2 3 11 12];
    GS_translate_str    = {'N','D','R','P','D-S1','D-E1'}; 
    
    for i = 1 : numel(GS_translate_num)
        CAN.gearstick_position_enum( ...
            CAN.gearstick_position == GS_translate_num(i) ...
            ) = GS_translate_str(i);
    end
    CAN.gearstick_position  = CAN.gearstick_position_enum;
    CAN(:,end)              = [];
    
    % Write to final table, only keep desired interval and return
	data_out            = CAN(...
        (CAN.Time >= t_start) & ...
        (CAN.Time <= t_end),:...
        );
end









