function [data_out]     = select_GPS(data,lags,t_start,t_end,Fs)
    global sFont figWidth figHeight figWidth_panel figHeight_panel cmap
    
    % List of selected GPS signals
    %   1 - speed
    %   2 - longitude
    %   3 - latitude
    
    % Optional
    %   Print figures showing GPS signal sampling distribution
    print_figure                = true;
    
    % Constant
    t_thresh    = 10;
    
    GPS         = data.DL004.Advantech_GPS.T_gps_raw;
    
    N           = size(GPS,1);
    
    % Speed limits
    vlim    = [2159 70;2488 50;2496 70;2602 100;2657 130; ...
        4233 100;4356 130;4943 120;5423 100;5852 130; ...
        7175 120;7546 100;7622 120;7645 130;8256 100; ...
        9384 80;11063 100;11202 100;12486 120;13224 120; ...
        13656 100;13880 130;14378 120;14437 80;14518 80; ...
        14578 70;15046 80;15094 100;15207 80;15229 100; ...
        15285 70;17141 70;17255 100;17313 80;17324 100; ...
        17450 80;17591 70;18010 80;18115 120;18516 130; ...
        21201 120;21382 100;22136 50];
    
    % Events
    %   - Traffic light
    %   - Invalid: tight corner, leaving valid roads etc.
    %       [approaching,   leaving];
    lights  = [2217 2224; 2302 2308; 2336 2342; 2356 2361; ...
        14816 14831; 14844 14849; 14882 14886; 14976 14987; ...
        15036 15041; 15221 15227; 15314 15364; 17202 17250; ...
        17320 17322; 17497 17588; 17635 17641; 17734 17743; ...
        17796 17802; 22193 22267; 22280 22284];
    invalid = [1 2159; 2488 2525; 2571 2582; 4325 4359; ...
        7582 7604; 9384 11063; 14437 14518; 14816 14831; ...
        14901 14934; 15417 17141; 17688 17703; 17813 17827; ...
        18080 18115; 19063 19085; 22061 22136; 22344 N];

    % Initiate tables
    GPS.v_lim           = zeros(N,1);
    GPS.traffic_lights 	= zeros(N,1);
    GPS.invalid         = zeros(N,1);

    for i = 1 : size(vlim,1)
        GPS.v_lim(vlim(i,1) : end) = vlim(i,2);
    end
    GPS.v_lim           = GPS.v_lim ./ 3.6;
    
    for i = 1 : size(lights,1)
        GPS.traffic_lights(lights(i,1) : lights(i,2)) = 1;
    end
    for i = 1 : size(invalid,1)
        GPS.invalid(invalid(i,1) : invalid(i,2)) = 1;
    end
    
    % Print figures
    if print_figure        
        %%%%%% First figure (sampling time distribution) %%%%%%
        hFig = figure();
        set(hFig, 'Position', [100 100 figWidth figHeight])
        set(gcf,'Color','white');

        hold on; grid minor
        tmp_dt  = milliseconds(diff(GPS.Time));
        tmp_dt(tmp_dt > 5000)  = NaN;
        tmp_dt_before   = tmp_dt;

        histogram(tmp_dt,'Normalization','Probability')

        title('Distribution of sampling times for the GPS signal (before correction)'); 
        xlabel('Time','FontSize',sFont,'FontWeight','normal');     
        ylabel('Probability, -','FontSize',sFont,'FontWeight','normal');
        set(gca,'FontSize',sFont)
        xlim([0 2500])

        %%%%%% Second figure (sampling time inconsistency) %%%%%%
        dt_small = find(tmp_dt > 300 & tmp_dt < 700);
        dt_medium = find(tmp_dt > 1300 & tmp_dt < 1750);
        dt_large = find(tmp_dt > 1750 & tmp_dt < 2050);

        hFig2 = figure();
        set(hFig2, 'Position', [100 100 figWidth figHeight])
        set(gcf,'Color','white');

        hold on; grid minor
        plot(GPS.Time(dt_small),ones(numel(dt_small),1),'*')
        plot(GPS.Time(dt_medium),ones(numel(dt_medium),1),'*')
        plot(GPS.Time(dt_large),ones(numel(dt_large),1),'*')

        legend('.4 - .6 s','1.4 - 1.6 s','1.9 - 2.05 s',...
            'Orientation','Horizontal','Location','North')        
        title('Patterns of sampling intervals over time'); 
        xlabel('Time','FontSize',sFont,'FontWeight','normal');     
        ylabel('Sampling intervals, -','FontSize',sFont,'FontWeight','normal');
        set(gca,'FontSize',sFont)
                  
        % Save old times for lon/lat plot
        tmp_time_old        = GPS.Time;
    end
    
    % Correct the GPS signal
    if numel(lags.t_lag_rmse) > 0
        % Compute the mean lags.t_lag_rmse
        t_lag_rmse_mean     = mean(lags.t_lag_rmse(~isnan(lags.t_lag_rmse)));

        % Write the mean lags.t_lag_rmse to all indices where NaN was reported
        lags.t_lag_rmse(isnan(lags.t_lag_rmse))     = t_lag_rmse_mean;

        % Find missing indices in the lags.t_lag_rmse signal
        [~,ia,~]            = intersect(GPS.Time,lags.Time);
        idxs_missing        = (1:size(GPS.Time,1))';
        idxs_missing(ia)    = [];

        % Adjust GPS signal
        %   1. For times present in the lags.t_lag_rmse signal
        GPS.Time(ia)            = GPS.Time(ia) + seconds(lags.t_lag_rmse)/100;
        %   2. For times not present in the lags.t_lag_rmse signal
        if numel(idxs_missing) > 0
            GPS.Time(idxs_missing)  = GPS.Time(idxs_missing) + seconds(t_lag_rmse_mean)/100;
        end
    end

    % Is print_figure is requested, do so...
    if print_figure
        %%%%%% Third figure %%%%%%
        hFig3 = figure();
        set(hFig3, 'Position', [100 100 figWidth figHeight])
        set(gcf,'Color','white');

        hold on; grid minor
        tmp_dt  = milliseconds(diff(GPS.Time));
        tmp_dt(tmp_dt > 5000)  = NaN;
        tmp_dt_after    = tmp_dt;

        histogram(tmp_dt,'Normalization','Probability')

        title('Distribution of sampling times for the GPS signal (after correction)'); 
        xlabel('Time','FontSize',sFont,'FontWeight','normal');     
        ylabel('Probability, -','FontSize',sFont,'FontWeight','normal');
        set(gca,'FontSize',sFont)
        xlim([0 2500])
        %%
        %%%%%% Fourth figure %%%%%%
        hFig4 = figure();
        set(hFig4, 'Position', [100 100 figWidth 0.5*figHeight])
        set(gcf,'Color','white');
        
        % Add plots to figure
        hold on; grid minor
        plot(lags.Time,lags.t_lag_rmse,'LineWidth',1.5)
        
        % Set subplot properties 
        title('Time-Lag of the GPS Signal'); 
        ylabel('Lag, ms','FontSize',sFont,'FontWeight','normal');
        set(gca,'FontSize',sFont)
        ylim([-150,-50])
        xlim([datetime([2018 06 13 10 18 00]), ...
            datetime([2018 06 13 10 20 00])])
        
        %% 
        %%%%%% Fifth figure %%%%%%
        dist_LL     = zeros(size(GPS.Time,1) + 1,1);
        lat         = GPS.latitude;
        lon         = GPS.longitude;
        for i = 2 : numel(GPS.Time)
            dist_LL(i)	= dist_LL(i-1) + ...
                lldistkm( ...
                    [lat(i-1), lon(i-1)], ...
                    [lat(i), lon(i)] ...
                    ) * 1000;
        end
        dist_LL     = dist_LL(2:end);
        
        hFig5 = figure();
        set(hFig5, 'Position', [100 100 figWidth figHeight])
        set(gcf,'Color','white');

        % Add plots to figure
        grid minor; hold on;
        plot(tmp_time_old,dist_LL,'LineWidth',1.5)
        plot(GPS.Time,dist_LL,'LineWidth',1.5)

        % Add properties to figure
        legend('x_{LL} (before correction)','x_{LL} (after correction)',...
            'Orientation','Horizontal','Location','North')
        title('Calculated driven distance from GPS coordinates')
        xlabel('Time','FontSize',sFont,'FontWeight','normal');
        ylabel('Distance, m','FontSize',sFont,'FontWeight','normal');
        set(gca,'FontSize',sFont)
        xlim([datetime([2018 06 13 08 16 27]),datetime([2018 06 13 08 16 48])]);
                
        %%
        %%%%%% Sixth figure %%%%%%
        hFig6    = figure();
        set(hFig6, 'Position', [100 100 figWidth_panel .7*figHeight_panel])
        set(gcf,'Color','white');
        
        dist_LL_retimed     = interp1(GPS.Time(2:end), ...
            dist_LL(1:end-1)-184189.3735, tmp_time_old, 'linear');
        
        % Initiate subplots
        p1             = panel();
        p1.pack('h', {1/2 1/2})
        
        p1(1).pack(1,1);
        p1(2).pack(2,1);

        % Add plots to subplot
        p1(1,1,1).select();
        grid minor; hold on;
        times_se = (lags.t_lag_rmse(1:end-1) - ...
            repelem(lags.t_lag_rmse(1:2:end-1),2))./100;
        plot(lags.Time(1:end-1), times_se, 'LineWidth', 1.5)
        plot(lags.Time(1:end-1), repelem(lags.t_lag_rmse(1:2:end-1),2) ./ 100, 'LineWidth', 1.5)

        % Add subplot properties
        legend('Sampling-Interval','Lag',...
            'Orientation','Horizontal','Location','North')
        ylabel('Correction, \Deltas','FontSize',sFont, ...
            'FontWeight','normal');
        set(gca,'FontSize',sFont)
        xlim_min    = datetime([2018 06 13 08 16 37]);
        xlim_max    = datetime([2018 06 13 08 16 58]);
        xlim([xlim_min, xlim_max]);
        set(gca,'XTick',xlim_min:seconds(10):xlim_max)
        datetick('x','HH:MM:ss','keeplimits','keepticks')
        
        % Add plots to subplot
        p1(2,1,1).select();
        grid minor; hold on;
        histogram(tmp_dt_before/1000,'Normalization','Probability','BinWidth',50/1000,'FaceAlpha', 1);

        % Add subplot properties
        set(gca,'FontSize',sFont)
        xlim([0 2.5])
        set(gca, 'YScale', 'log')
        ylim([10^-3 10^0])
        set(gca,'XTickLabel',[]);

        % Add plots to subplot
        p1(2,2,1).select();
        grid minor; hold on;
        histogram(tmp_dt_after/1000,'Normalization','Probability','BinWidth',50/1000,'FaceAlpha', 1);

        % Add subplot properties
        ylab = ylabel('Probability, -','FontSize',sFont, ...
            'FontWeight','normal', 'Units', 'Normalized');
        ylab.Position(2) = 1.2;
        set(gca,'FontSize',sFont)
        xlim([0 2.5])
        set(gca, 'YScale', 'log')
        ylim([10^-3 10^0])
        
        % Add figure properties
        p1(2).xlabel('Sampling Interval, s');
        p1.margin = [13 13 5 5];      % Outside
        p1(1).fontsize     = sFont;
        p1(1).fontweight   = 'normal';
        p1(1).margin = [1 0 0 0];      % Outside
        p1(1).de.margin = 1;            % Internal
        p1(2).fontsize     = sFont;
        p1(2).fontweight   = 'normal';
        p1(2).margin = [15 18 0 0];      % Outside
        p1(2).de.margin = 3;            % Internal

end
    
    % Remove inconsistencies 
    GPS         = GPS(abs(GPS.longitude)<360,:);
    GPS         = GPS(abs(GPS.longitude)<360,:);
    
    % Only keep the desired fields and write VariableContinuity
    TT          = GPS(:,ismember(fieldnames(GPS),{'speed','longitude','latitude','v_lim','traffic_lights','invalid'}));
    TT.Properties.VariableContinuity = {'continuous','continuous','continuous','step','step','step'};

    GPS_out     = timetable();
    GPS_out     = add_to_timetable(GPS_out,TT,Fs,t_thresh);

    % Only keep desired time interval and output data
	data_out            = GPS_out(...
        (GPS_out.Time >= t_start) & ...
        (GPS_out.Time <= t_end),:...
        );
end









