function [p_ACC_syn, data_out, varargout] = ...
    create_synthetic_data_ACC(data_in, Np, plot_figures, varargin)
    global sFont figWidth figHeight

    % Check if kp, kd, td values are provided as an input, otherwise use
    %   global variables
    if numel(varargin) == 0
        global kp_range kd_range td_range
    else
        kp_range    = repmat(varargin{1}(1),1,2);
        kd_range    = repmat(varargin{1}(2),1,2);
        td_range    = repmat(varargin{1}(3),1,2);
    end
    global calibration_setup
    
    % Make sure all random sequences are repeatable
    rng(0,'twister');
    
    % Define parameter ranges
    kp_vec      = kp_range(1) + (kp_range(2)-kp_range(1)).*rand(Np,1);
    kd_vec      = kd_range(1) + (kd_range(2)-kd_range(1)).*rand(Np,1);
    td_vec      = td_range(1) + (td_range(2)-td_range(1)).*rand(Np,1);
    
    % Find all parameter combinations
    combs       = permn(1:Np, 3);
    p_ACC_syn   = [kp_vec(combs(:,1)), kd_vec(combs(:,2)), td_vec(combs(:,3))];
    
    % Initiate final table
    tmp_data_out    = data_in(:,ismember(fieldnames(data_in), {'reset','reset_type'}));
    tmp_data_out.xl = data_in.x - data_in.x(1);
    tmp_data_out.vl = data_in.v;
    N               = size(data_in.x);

    % Compute follower trajectories
    data_out        = struct();

    for i = 1 : size(p_ACC_syn,1)
        x   = zeros(N);
        v	= zeros(N);
        s 	= zeros(N);
        
        % Calculate vectors at reset points 
        %   s_opt = argmin_s a(s | v_{i}=v_{i-1})
        % data_out.s      = data_out.s .* data_in.reset;
        i_reset   	= find(data_in.reset)';
        for j = i_reset
            v(j)	= tmp_data_out.vl(j);
            s(j) 	= find_s_opt(p_ACC_syn(i,:), v(j), 'ACC');
            x(j)  	= tmp_data_out.xl(j) - s(j);
        end
        
        % Generate trajectories
        [a_model,v_model,x_model,s_model,~]  = ...
                get_model_trajectories_ACC(false, ...
                p_ACC_syn(i,:), ...
                x, ...
                v, ...
                tmp_data_out.xl, ...
                tmp_data_out.vl, ...
                s, ...
                tmp_data_out.reset, ...
                tmp_data_out.reset_type);
            
        subset_title                        = strcat('traject',num2str(i));
        data_out.(subset_title).timetab     = tmp_data_out;
        data_out.(subset_title).timetab.a   = a_model;
        data_out.(subset_title).timetab.v 	= v_model;
        data_out.(subset_title).timetab.x 	= x_model;
        data_out.(subset_title).timetab.s  	= s_model;
        data_out.(subset_title).calibration_setup   = calibration_setup;
    end
        
    if plot_figures
        FN          = fieldnames(data_out);
        for i = 1 : numel(FN)
            hFig = figure();
            set(hFig, 'Position', [100 100 figWidth figHeight])
            set(gcf,'Color','white');

            sp1 = subplot(2,1,1);
            hold on; grid minor
            plot(data_out.(FN{i}).timetab.Time, ...
                data_out.(FN{i}).timetab.vl,'LineWidth',1.5);
            plot(data_out.(FN{i}).timetab.Time, ...
                data_out.(FN{i}).timetab.v,'LineWidth',1.5);

            title(char(strcat('Generation model: ACC ([',...
                num2str(round(p_ACC_syn(i,1),2)),{', '}, ...
                num2str(round(p_ACC_syn(i,2),2)),{', '}, ...
                num2str(round(p_ACC_syn(i,3),2)),'])'))); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel('Speed, m/s','FontSize',sFont,'FontWeight','normal');
            legend('$$v_{i-1}$$','$$v_{i}$$','Interpreter','Latex', ...
                'Orientation','Horizontal','Location','North')
            set(gca,'FontSize',sFont)

            sp2 = subplot(2,1,2);
            hold on; grid minor
            plot(data_out.(FN{i}).timetab.Time, ...
                data_out.(FN{i}).timetab.s,'LineWidth',1.5);

            title('Distance-gap'); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel('Distance-gap, m','FontSize',sFont,'FontWeight','normal');
            legend('$$s_{i}$$','Interpreter','Latex', ...
                'Orientation','Horizontal','Location','North')
            set(gca,'FontSize',sFont)

            linkaxes([sp1,sp2],'x')
            
            varargout{1}    = hFig;
            varargout{2}    = sp1;
            varargout{3}    = sp2;
        end
    end
    varargout{4}    = 'NA';
end







