function [p_IDM_syn, data_out, varargout] = ...
    create_synthetic_data_IDM(data_in, Np, plot_figures, varargin)
    global sFont figWidth figHeight
    
    % Check if a, b, T, v0, s0 values are provided as an input, otherwise use
    %   global variables
    if numel(varargin) == 0
        global a_range b_range T_range v0_range s0_range
    else
        a_range     = repmat(varargin{1}(1),1,2);
        b_range     = repmat(varargin{1}(2),1,2);
        T_range     = repmat(varargin{1}(3),1,2);
        v0_range    = repmat(varargin{1}(4),1,2);
        s0_range    = repmat(varargin{1}(5),1,2);
    end
    global calibration_setup
    
    % Make sure all random sequences are repeatable
    rng(0,'twister');
    
    % Define parameter ranges
    a_vec       = a_range(1) + (a_range(2)-a_range(1)).*rand(Np,1);
    b_vec       = b_range(1) + (b_range(2)-b_range(1)).*rand(Np,1);
    T_vec       = T_range(1) + (T_range(2)-T_range(1)).*rand(Np,1);
    v0_vec      = v0_range(1) + (v0_range(2)-v0_range(1)).*rand(Np,1);
    s0_vec      = s0_range(1) + (s0_range(2)-s0_range(1)).*rand(Np,1);
    
    % Find all parameter combinations
    combs       = permn(1:Np, 5);
    p_IDM_syn   = [a_vec(combs(:,1)), b_vec(combs(:,2)), ...
        T_vec(combs(:,3)), v0_vec(combs(:,4)), s0_vec(combs(:,5))];
    
    % Initiate final table
    tmp_data_out    = data_in(:,ismember(fieldnames(data_in), {'reset', 'reset_type'}));
    tmp_data_out.xl = data_in.x - data_in.x(1);
    tmp_data_out.vl = data_in.v;
    N               = size(data_in.x);
    
    % Compute follower trajectories
    data_out        = struct();

    for i = 1 : size(p_IDM_syn,1)
        x   = zeros(N);
        v	= zeros(N);
        s 	= zeros(N);
        
        % Calculate vectors at reset points 
        %   s_opt = argmin_s a(s | v_{i}=v_{i-1})
        % data_out.s      = data_out.s .* data_in.reset;
        i_reset   	= find(data_in.reset)';
        for j = i_reset
            v(j)	= tmp_data_out.vl(j);
            s(j) 	= find_s_opt(p_IDM_syn(i,:), v(j), 'IDM');
            x(j)  	= tmp_data_out.xl(j) - s(j);
        end
        
        % Generate trajectories
        [a_model,v_model,x_model,s_model,~]  = ...
                get_model_trajectories_IDM(false, ...
                p_IDM_syn(i,:), ...
                x, ...
                v, ...
                tmp_data_out.xl, ...
                tmp_data_out.vl, ...
                s, ...
                tmp_data_out.reset, ...
                tmp_data_out.reset_type);
            
        subset_title                        = strcat('traject',num2str(i));
        data_out.(subset_title).timetab     = tmp_data_out;
        data_out.(subset_title).timetab.a   = a_model;
        data_out.(subset_title).timetab.v 	= v_model;
        data_out.(subset_title).timetab.x 	= x_model;
        data_out.(subset_title).timetab.s  	= s_model;
        data_out.(subset_title).calibration_setup = calibration_setup;
    end
    
    if plot_figures
        FN          = fieldnames(data_out);
        for i = 1 : numel(FN)
            hFig = figure();
            set(hFig, 'Position', [100 100 figWidth figHeight])
            set(gcf,'Color','white');

            sp1 = subplot(2,1,1);
            hold on; grid minor
            plot(data_out.(FN{i}).timetab.Time, ...
                data_out.(FN{i}).timetab.vl,'LineWidth',1.5);
            plot(data_out.(FN{i}).timetab.Time, ...
                data_out.(FN{i}).timetab.v,'LineWidth',1.5);

            title(char(strcat('Generation model: IDM ([',...
                num2str(round(p_IDM_syn(i,1),2)),{', '}, ...
                num2str(round(p_IDM_syn(i,2),2)),{', '}, ...
                num2str(round(p_IDM_syn(i,3),2)),{', '}, ...
                num2str(round(p_IDM_syn(i,4),2)),{', '}, ...
                num2str(round(p_IDM_syn(i,5),2)),'])'))); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel('Speed, m/s','FontSize',sFont,'FontWeight','normal');
            legend('$$v_{i-1}$$','$$v_{i}$$','Interpreter','Latex', ...
                'Orientation','Horizontal','Location','North')
            set(gca,'FontSize',sFont)

            sp2 = subplot(2,1,2);
            hold on; grid minor
            plot(data_out.(FN{i}).timetab.Time, ...
                data_out.(FN{i}).timetab.s,'LineWidth',1.5);

            title('Distance-gap'); 
            xlabel('Time','FontSize',sFont,'FontWeight','normal');     
            ylabel('Distance-gap, m','FontSize',sFont,'FontWeight','normal');
            legend('$$s_{i}$$','Interpreter','Latex', ...
                'Orientation','Horizontal','Location','North')
            set(gca,'FontSize',sFont)

            linkaxes([sp1,sp2],'x')
            
            varargout{1}    = hFig;
            varargout{2}    = sp1;
            varargout{3}    = sp2;
        end
    end
    varargout{4}    = 'NA';
end







