function [err] = plot_model_response_noise(data_nonoise, data, varargin)
    global figWidth_panel figHeight_panel sFont
    % This script is actually almost equal to the plot_model_response
    % script, with the only difference being that noisy measurements are
    % plotted next to the non-noisy measurements.
    
    % Varargin contains the desired data to plot:
    %   - Leader trajectories
    %   - Desired model for follower trajectory generation
    %   - Considered parameter set
    %   varargin{i:i+2}     = ...
    %       { <<model: IDM/ACC/ACC_alternative>>, ...
    %       <<parameters>>, ...
    %       <<legend label>> }
    
    % The model outputs the representation error of the last provided
    % model-parameter value pair
    
    %% Plot response captured in actual data
    % Create multiple subplots in case large discontinuities are found
    % (larger than 10 seconds)
    dt_data         = diff(data.Time);
    idx_missing     = [0; find(dt_data > seconds(10)); size(data,1)];
    idx_plot        = [idx_missing(1:end-1) + 1, idx_missing(2:end)];
    N_subplots      = size(idx_plot,1);
    t_parts         = NaN(N_subplots,1);
    for i = 1 : N_subplots
        t_parts(i)	= seconds(data.Time(idx_plot(i,2)) - data.Time(idx_plot(i,1)));
    end
    size_parts      = t_parts ./ sum(t_parts);
    
    hFig = figure();
    set(hFig, 'Position', [100 100 .5*figWidth_panel figHeight_panel])
    set(gcf,'Color','white');
    
    p1              = panel();
    p1.pack(2,num2cell(size_parts'))    
    p1.fontsize     = sFont;
    p1.fontweight   = 'normal';
    p1.margin       = [12 11 5 6];  % Outside
    p1.de.margin    = 1;            % Internal
    p1.de.margintop = 3;            % Internal

    for j = 1 : N_subplots
        idxs        = idx_plot(j,1) : idx_plot(j,2);
        
        hAx(1,j) 	= p1(1,j).select();
        hold on; grid minor
        plot(data.Time(idxs), data.v(idxs),'LineWidth',1.5);
        plot(data.Time([1 2]), [NaN NaN],'LineWidth',1.5);
        set(gca,'XTickLabel',[])

        hAx(2,j)	= p1(2,j).select();
        hold on; grid minor
        plot(data_nonoise.Time(idxs), data_nonoise.s(idxs),'LineWidth',1.5);
        plot(data.Time(idxs), data.s(idxs),'LineWidth',1.5);
        set(gca,'XTickLabels',[]);
    end

    %% Get model trajectories from input parameters
    v_model     = NaN(numel(varargin)/3,size(data,1));
    s_model     = NaN(numel(varargin)/3,size(data,1));
    
    % Obtain model trajectories and add to plot for each of the desired
    % model/parameter value-pairs
    for i = 1 : 3 : numel(varargin)
        if strcmpi(varargin{i},'IDM')
            [~,v_model(i,:),~,s_model(i,:),err] = ...
                get_model_trajectories_IDM(false, ...
                varargin{i+1}, ...
                data.x, ...
                data.v, ...
                data.xl, ...
                data.vl, ...
                data.s, ...
                data.reset, ...
                data.reset_type);
        elseif strcmpi(varargin{i},'ACC')
            [~,v_model(i,:),~,s_model(i,:),err] = ...
                get_model_trajectories_ACC(false, ...
                varargin{i+1}, ...
                data.x, ...
                data.v, ...
                data.xl, ...
                data.vl, ...
                data.s, ...
                data.reset, ...
                data.reset_type);
        elseif strcmpi(varargin{i},'ACC_alternative')
            [~,v_model(i,:),~,s_model(i,:),err] = ...
                get_model_trajectories_ACC_alt(false, ...
                varargin{i+1}, ...
                data.x, ...
                data.v, ...
                data.xl, ...
                data.vl, ...
                data.s, ...
                data.reset, ...
                data.reset_type);
        else
            fprintf('Error, no model specified.\n')
            return
        end
        for j = 1 : N_subplots
            % Find reset indices
            idxs        = idx_plot(j,1) : idx_plot(j,2);
            idx_reset   = find(data.reset(idxs));
            idx_reset   = idxs(idx_reset);

            p1(1,j).select();
            plot(data.Time(idxs), v_model(i,idxs),'LineWidth',1.5);
            if i == (numel(varargin) - 2)
                plot(data.Time(idx_reset), v_model(i,idx_reset),'*');
            end

            p1(2,j).select();
            plot(data.Time(idxs), s_model(i,idxs),'LineWidth',1.5);
            if i == (numel(varargin) - 2)
                plot(data.Time(idx_reset), s_model(i,idx_reset),'*');
            end
        end
    end
    
    %% Finalise figure
    p1(1,1).select();
    ylabel('$$v, m/s$$','FontSize',sFont,'FontWeight','normal', ...
        'Interpreter','Latex');
    set(gca,'XTickLabels',[]);


    p1(2,1).select();
    ylabel('$$s, m/s$$','FontSize',sFont,'FontWeight','normal', ...
        'Interpreter','Latex');
    set(gca,'XTickLabels',[]);

    set(hAx,'XTick', [dateshift(data.Time(1), 'start', 'minute') : seconds(10) : ...
        dateshift(data.Time(end), 'end', 'minute')]);
    datetick('x','HH:MM:ss','keeplimits','keepticks')    
    
    for i = size(hAx,2)
        p1(1,i).select();
        lgnd    = legend('Data','Data (noisy)', ...
            varargin{3:3:end},'Reset','Interpreter','Latex', ...
            'Orientation','Horizontal');
        lgnd.ItemTokenSize  = [15,18];
        lgnd.Position       = [(1-lgnd.Position(3))/2,0.913, ...
            lgnd.Position(3:4)];
    end
    
    for i = 2 : size(hAx,2)
        set(hAx(:,i),'YTickLabels',[])
    end
    
    linkaxes(hAx,'x')
    xlim([datetime([2018 06 13 14 16 30]), datetime([2018 06 13 14 16 52])])
    
    set(hAx,'YLimSpec','Tight')
end











