function [err_cali] = plot_fitness_landscape_ACC(AUDI, data_in, N_points, ...
    generation_model)
    global p_ACC_pub p_IDM_pub
    global sFont figWidth_panel figHeight_panel cmap
    global calibration_setup dt_data
    
    load('CustomMap', '-mat')
    
    % Create follower-trajectory
    if strcmpi(generation_model,'ACC')
        [~, data_syn, ~, ~, ~] = ...
            create_synthetic_data_ACC(data_in, 1, false, p_ACC_pub);
    elseif strcmpi(generation_model,'IDM')
        [~, data_syn, ~, ~, ~] = ...
            create_synthetic_data_IDM(data_in, 1, false, p_IDM_pub);
    end
    
    % Add noise to trajectories
    noise           = str2double(strsplit(calibration_setup.noise,'-'));
    noise_type      = calibration_setup.noise_type;
    noise_mean      = noise(1);
    noise_variance  = noise(2);
    
    data_noisy      = add_noise_to_trajectory(data_syn, ...
        noise_type, noise_mean, noise_variance);
    
    % Add extra reset points
    t_reset         =  calibration_setup.t_reset;
    reset_idxs      = [find(data_noisy.traject1.timetab.reset == 1); 
        numel(data_noisy.traject1.timetab.reset)+1];
    i_reset         = max(1,t_reset / dt_data);
    for j = 1 : numel(reset_idxs) - 1
        idxs        = reset_idxs(j) : reset_idxs(j+1) - 1;
        data_noisy.traject1.timetab.reset(idxs(1) : i_reset : idxs(end)) = 1;
    end
    data_out = data_noisy;

    % Calibrate model to synthetic data
    [p_cali, err_cali, exitflag, ~] = calibrate_ACC(data_noisy);

    % Initiate fitness landscape vectors
    landscape           = [];    landscape.kp_vec    = linspace(max(0.02,min(0.02,p_cali(1)*0.5)),  max(0.4,p_cali(1)*1.5),N_points);
    landscape.kd_vec 	= linspace(max(0.03,min(0.01,p_cali(2)*0.5)),  max(.6,p_cali(2)*1.5),N_points);
    landscape.T_vec   	= linspace(min(.4,p_cali(3)*0.5),  max(1.8,p_cali(3)*1.5),N_points);
    
    %% Plot model response

    FN  = fieldnames(data_noisy);
    plot_model_response(AUDI, data_noisy.(FN{1}).timetab, 'ACC', ...
        p_cali, '$$p_{cali}$$')
    plot_model_response_noise(data_syn.(FN{1}).timetab, ...
        data_noisy.(FN{1}).timetab, ...
        'ACC', p_cali, 'ACC')
    
    %% Create fitness landscape
    FN_LS   = fieldnames(landscape);
    var_labels = {'$${k_p, s^{-2}}$$', '$${k_d, s^{-1}}$$', '$${t_d, s}$$'};
    N       = numel(FN_LS);

    hFig2 = figure();
    set(hFig2, 'Position', [100 100 figWidth_panel figHeight_panel])
    set(gcf,'Color','white');

    p2              = panel();
    p2.pack(N-1,N-1)
    p2.fontsize     = sFont;
    p2.fontweight   = 'normal';
    
    hFig3 = figure();
    set(hFig3, 'Position', [100 100 figWidth_panel 0.5*figHeight_panel])
    set(gcf,'Color','white');
    
    p3              = panel();
    p3.pack(1,N)
    p3.fontsize     = sFont;
    p3.fontweight   = 'normal';
    
    progressbar(0,0,0,0)
    for i = 1 : N
        for j = i : N
            % Current x and y-axes for the landscape plot
            x_vec   = landscape.(FN_LS{i});
            y_vec   = landscape.(FN_LS{j});
            err     = NaN(numel(y_vec),numel(x_vec));

            % Find all values for the landscape
            for m = 1 : numel(x_vec)
                for n = 1 : numel(y_vec)
                    if rem(n,10) == 0 || n == 0
                        progressbar((i-1)/N,(j-1)/N,(m-1)/N_points,(n-1)/N_points)
                    end

                    p       = p_cali;
                    p(i)    = x_vec(m);
                    p(j)    = y_vec(n);

                    [~,~,~,~,err(n,m)] = ...
                        get_model_trajectories_ACC(false, ...
                            p, ...
                            data_out.(FN{1}).timetab.x, ...
                            data_out.(FN{1}).timetab.v, ...
                            data_out.(FN{1}).timetab.xl, ...
                            data_out.(FN{1}).timetab.vl, ...
                            data_out.(FN{1}).timetab.s, ...
                            data_out.(FN{1}).timetab.reset, ...
                            data_out.(FN{1}).timetab.reset_type);
                end
            end      
            figlist     = [hFig2,hFig3];

            %% Landscape plot
            Np      = 80;       % Maximum percentile to show

            % Plot the actual landscape
            if i < j && j > 1 && i ~= j
                for k = 1
                    p2(j-1,i).select();
                    hold on; grid minor;
                    iplot = imagesc(x_vec,y_vec,err);
                    contour(x_vec,y_vec, ...
                        err, prctile(err(:),10:10:Np)', ...
                        'k','HandleVisibility','off')
                    plot(p_cali(i),p_cali(j),'*','MarkerEdgeColor',cmap(1,:))
                        plot(p_ACC_pub(i),p_ACC_pub(j),'*','MarkerEdgeColor',cmap(2,:))

                    hcb = colorbar;
                    if i == 1 && j == 2
                        colorTitleHandle = get(hcb,'Label');
                        titleString = '$$|\mathcal{F}|$$';
                        set(colorTitleHandle,'Interpreter','latex','String',titleString);
                    end
                    colormap(mymap)
                    set(gca,'FontSize',sFont)
                    caxis([-Inf, prctile(err(:),Np)])
                    
                    if j == N
                        p2(j-1,i).select();
                        xlabel(var_labels{i}, ...
                            'FontSize',sFont,'FontWeight','normal','Interpreter', 'Latex');
                    else
                        p2(j-1,i).select();
                        set(gca,'xtick',[]); 
                    end
                    if i == 1
                        p2(j-1,1).select();
                        ylabel(var_labels{j}, ...
                            'FontSize',sFont,'FontWeight','normal','Interpreter', 'Latex');
                    else
                        p2(j-1,i).select();
                        set(gca,'YTickLabels',[]);
                    end
                    if i == 1 && j == 2
                        lgnd = legend('Calibrated','Published', ...
                            'Orientation','Horizontal','Location','North');
                        lgnd.ItemTokenSize = [14,18];
                        lgnd.Position = [(1-lgnd.Position(3))/2,0.901766518324408, ...
                            lgnd.Position(3:4)];
                    end
                end
            end
            
            %% Parameter sensitivity plot
            if i == j 
                p3(1,i).select();
                xlabel(var_labels{i}, ...
                    'FontSize',sFont,'FontWeight','normal','Interpreter', 'Latex');

                hold on; grid minor;
                plot(x_vec,err(:,1),'LineWidth',1.5,'Color',cmap(6,:))
                plot(p_cali(i),min(err(:,1)),'*','MarkerEdgeColor',cmap(1,:))
     
                plot(p_ACC_pub(i),min(err(:,1)),'*','MarkerEdgeColor',cmap(2,:))
                xlim([x_vec(1) x_vec(end)])
                
                % Print parameter sensitivity
                [~,idx]         = min(abs(p_cali(i)-x_vec));
                [~,tmp_idx_min] = min(abs(x_vec(idx) * 0.9 - x_vec));
                idx_min         = max(min(tmp_idx_min, idx-1),1);
                [~,tmp_idx_max] = min(abs(x_vec(idx) * 1.1 - x_vec));
                idx_max         = min(max(tmp_idx_max, idx+1),numel(x_vec));
                ttl = title({char(strcat({'$$S^-$$ = '}, ...
                    num2str(round((err(idx_min,1) - err(idx,1)) / (x_vec(idx) - x_vec(idx_min)),2)))), ...
                    char(strcat({'$$S^+$$ = '}, ...
                    num2str(round((err(idx_max,1) - err(idx,1)) / (x_vec(idx_max) - x_vec(idx)),2)) ...
                ))}, 'FontWeight','Normal','Interpreter','Latex',...
                'Units','Normalized');
                ttl.Position(2)     = 0.2;
                
                if i == N && j == N
                    lgnd = legend('Error','Calibrated','Published', ...
                        'Orientation','Horizontal','Location','North');
                    lgnd.ItemTokenSize = [14,18];
                    lgnd.Position = [(1-lgnd.Position(3))/2,0.821766518324408, ...
                        lgnd.Position(3:4)];
                end

                if i == 1
                    p3(1,1).select();
                    ylabel(char(strcat({'$$\mathcal{F}_{'}, ...
                        calibration_setup.error_measure, ...
                        {'}$$'})), ...
                        'FontSize',sFont,'FontWeight','normal','Interpreter', 'latex');
                else
                    p3(1,i).select();
                    set(gca,'YTickLabels',[]);
                end
            end
        end
    end
    figure(figlist(1))
    p2.de.margin = 1;            % Internal
    p2.de.marginright = 15;            % Internal
    p2.de.margintop = 3;            % Internal    
    p2.margin = [12 11 15 6];      % Outside

    figure(figlist(2))
    allAxesInFigure = findall(figure(figlist(2)),'type','axes');
    YLims           = get(allAxesInFigure,'YLim');
    YLim_all        = [YLims{:}];
    YMin_max        = min(YLim_all(2:2:end));
    linkaxes(allAxesInFigure,'y')
    set(allAxesInFigure,'YLim',[-Inf, min(YMin_max,10)])
    set(allAxesInFigure,'XLimMode','auto')
    
    p3.margin = [14 11 5 5];      % Outside
    p3.de.margin = 5;            % Internal
    
    progressbar(1,1,1,1)
end
















