function [hFig, p] = plot_parameter_distribution(p_cali, p_variations, ...
    p_choice, p_names, labels, type, varinfo, model, varargin)
    global sFont figWidth figHeight figWidth_panel figHeight_panel 
    
    %% Compute error of parameter estimates

    % Compute parameter calibration error
    if strcmpi(p_choice,'rel')
        p_err   = (p_cali - p_variations) ./ p_variations;
        ysca    = 'linear';
        ylab    = '$$(\beta_i^*-\hat{\beta_i})/\hat{\beta_i}$$';
    elseif strcmpi(p_choice,'absrel')
        p_err    = abs((p_cali - p_variations) ./ p_variations);
        ysca     = 'log';
        ylab    = '$$|(\hat{\beta_i} - \beta_i^*) / \beta_i^*|$$';
    end
    
    %% Rearrange for length variations
    if strcmpi(type,'length')
        labels_unique   = sort(unique(str2double(labels)),'descend');

        tmp_p_values    = [];
        tmp_p_labels    = [];
        
        for i = 1 : numel(labels_unique)
            tmp_p_curr  = p_err(strcmpi(labels,num2str(labels_unique(i))),:,:);
            p_curr      = permute(tmp_p_curr,[3,1,2]);
            times       = varinfo.times{i};
            p_curr      = p_curr(1:numel(times),:,:);   % Only keep valid p  
            for j = 1 : size(p_err,2)
                tmp_p_curr      = p_curr(:,:,j);
                tmp_p_curr      = tmp_p_curr(:);

                if j == 1
                    tmp_p_values    = [tmp_p_values; NaN(numel(tmp_p_curr), size(p_err,2))];
                end
                tmp_p_values(end-(numel(tmp_p_curr)-1):end,j) = tmp_p_curr;
            end
            tmp_p_labels    = [tmp_p_labels; repmat(labels_unique(i),numel(tmp_p_curr),1)];
        end
        
        p_err   = tmp_p_values;
        labels  = mat2cell(num2str(tmp_p_labels),ones(1,size(tmp_p_labels,1),1));
    end

    %% Plot result using panel
    if isempty(varargin)
        hFig = figure();
        set(hFig, 'Position', [100 100 figWidth_panel figHeight_panel])
        set(gcf,'Color','white');
        
        p       = panel();
        p.pack(2,1)
        for i = 1 : 2
            p(i).pack('h',{1/5 1/5 1/5 1/5});
        end
        
        p.fontsize = sFont;
        p.fontweight = 'normal';
        
        row     = 1;
    else
        hFig    = varargin{1};
        p       = varargin{2};
        row     = 2;
    end
    
    % Make note if labels are numeric
    label_is_num    = false;
    labels_num      = str2num(char(labels));
    if sum(strcmpi(type,{'reset','length','noise_L','noise_FL'}))
        label_is_num    = true;
        labels          = labels_num;
    end
    
    % Create boxplots
    %   MATLAB automatically splits the boxplots over different label
    %   values
    clear hAx bp
    YLim_min    = Inf;
    YLim_max    = 0;
    for i = 1 : numel(p_names)
        hAx(i)      = p(row,i).select();
        hold on; grid minor
        if label_is_num
            bp          = boxplot(p_err(:,i), log10(labels), ...
                'Positions', unique(log10(labels)), 'symbol', '', ...
                 'Widths',.4);
        else
            bp          = boxplot(p_err(:,i), labels, 'symbol', '');
        end
        YMin        = get(bp(4:7:end),'YData'); YMin = [YMin{:}];
        YMax        = get(bp(3:7:end),'YData'); YMax = [YMax{:}];
        YLim_min    = min(min(YMin),YLim_min);
        YLim_max    = max(max(YMax),YLim_max);
        
        title(char(strcat('$$',p_names{i},'$$')),'Interpreter','Latex')
        
        % Change XTicks and XTickLabels
        if (row == 1) && (ismember(i,1:3))
            set(gca,'XTickLabels',[])
        else
            if label_is_num
                set(gca,'Xtick',unique(log10(labels))); %// adjust manually; values in log scale
                set(gca,'XTickLabel',num2str(10.^get(gca,'Xtick')')); %// use labels with linear values
            end
        end
        
        % Rotate ytick labels and change position.
        xtickangle(90)
        if (i == 1) && (~isempty(varargin))
            ylab = ylabel(ylab,'FontSize',sFont,'FontWeight','normal', ...
                'Interpreter','Latex', 'Units', 'Normalized');
            ylab.Position(1)    = -0.4;
            ylab.Position(2)    = 1.13;
        elseif i == 1
            
        else
            set(gca,'YTickLabels',[]);
        end
        set(gca,'YScale',ysca)
        set(gca,'YTick', 10.^(-12:2:0))
    end
    
    linkaxes(hAx,'y')
    set(hAx,'YLim',[0.2*YLim_min 5*YLim_max])

    p.margin = [16 15 8 5];      % Outside

    p.de.margin     = 1;            % Internal
    p.de.margintop  = 5;            % Internal
    
    % Annotate subplots to save space in Latex
    if row == 1
        annotation('textbox', [0.95, .85, 0, 0], 'string', '(a)', ...
            'FontWeight', 'Normal')
    else
        annotation('textbox', [0.95, .45, 0, 0], 'string', '(b)', ...
            'FontWeight', 'Normal')
    end
end









