%% This script is written to fuse measurement signals using a Kalman filter

% Script written by:    Mathieu Blauw
% Date:                 April 2nd, 2019
% Institution:          Delft University of Technology

%% Clear variables and add functions to current path
clearvars -except tab_all GPS_lag delta_k
addpath('./00_function_files_exchange/progressbar')                      % https://nl.mathworks.com/matlabcentral/fileexchange/6922-progressbar
addpath('./00_function_files_exchange/google_maps')                      % https://nl.mathworks.com/matlabcentral/fileexchange/27627-zoharby-plot_google_map
addpath('./00_function_files_exchange/lonlat_to_km')                     % https://nl.mathworks.com/matlabcentral/fileexchange/38812-latlon-distance
addpath('./00_function_files_exchange/confplot')                         % https://nl.mathworks.com/matlabcentral/fileexchange/2683-confplot
addpath('./00_function_files_exchange/panel')                            % https://nl.mathworks.com/matlabcentral/fileexchange/20003-panel
addpath('./00_function_files_exchange/export_fig')                       % https://nl.mathworks.com/matlabcentral/fileexchange/23629-export_fig
    % export_fig('test_image.pdf', '-pdf', '-c[NaN,NaN,NaN,NaN]', '-transparent', '-CMYK')
addpath('./00_function_files_synthetic_real')
addpath('./00_function_files_pre_analysis')
addpath('./02_function_files_kalman')

%% Load data if not already existing ... 
if ~exist('tab_all','Var')
    tic
    load(fullfile('E:\Graduation\SAE-L2\Matlab','data_audi.mat'))
    t_load  = toc;
    fprintf(char(strcat({'Loaded data in '},num2str(t_load),{' seconds.\n'})))
else
    fprintf('Data already loaded. Continue without loading ...\n')
end
if ~exist('lags','Var')
    load(fullfile('../a2. output','lags_with_correction_v3.mat'))
end
if ~exist('delta_k','Var')
    load(fullfile('../a2. output','delta_k_v2.mat'))
end

%% Initiate ...

% ... figure properties
global sFont figWidth figHeight figWidth_panel figHeight_panel cmap
sFont       = 10;
figWidth    = 660;
figWidth_panel  = 600;
figHeight       = 250;
figHeight_panel = 225;
figure(100)
cmap        = colormap(lines(10));
close(figure(100))

% ... constants
filterWidth_ego_v   = 4;

g  	= 9.80665;
Fs	= 10;

%% Select data

% Desired time-interval to keep
t_start     = datetime([2018 06 13 00 00 00]);
t_end       = datetime([2018 06 14 00 00 00]);

% Select data
GPS         = select_GPS(tab_all,lags,t_start,t_end,Fs);
fprintf('Loaded: GPS\n')
CAN         = select_CAN(tab_all,t_start,t_end,Fs);
CAN.vehicle_speed   = CAN.vehicle_speed * (1-delta_k);
fprintf('Loaded: CAN\n')
ACL         = select_ACL(tab_all,t_start,t_end,Fs);
fprintf('Loaded: ACL\n')

% Only keep common times
times       = intersect(GPS.Time,intersect(CAN.Time,ACL.Time));
GPS         = GPS(ismember(GPS.Time, times),:);
CAN         = CAN(ismember(CAN.Time, times),:);
ACL         = ACL(ismember(ACL.Time, times),:);

% Find missing data (reset indices)
gap_idxs    = [0;find(diff(GPS.Time) > seconds(2));numel(GPS.Time)];

%% Perform initial optimisation process noise covariance matrix Q
optim_kal   = input('Optimize Kalman filter? Yes(1)/No(0)');

if optim_kal
    fun_kal     = @(q) obj_fun_kalman(q, GPS.Time, CAN.vehicle_speed, GPS.speed, ACL.x, gap_idxs(17:18));

    A           = [];
    b           = [];
    Aeq         = [];
    beq         = [];
    lb          = [0    0   10  0];
    ub          = [10   1   25  1.5];
    nonlcon     = [];
    options     = optimoptions('ga','Display','iter','UseParallel',1,...
        'MaxTime',1800);

    [q,costval,exitflag,output] = ...
        ga(fun_kal,4,A,b,Aeq,beq,lb,ub,nonlcon,options);
else % if load Q from file ...
    load('./02_function_files_kalman/q_optimum.mat')
end

%% Apply Kalman filter

% Initiate covariance matrices Q and R
q11     = q(1); 
q22     = q(2); 
q33     = q(3); 
q44     = q(4); 
r11     = (4).^2; 
r22     = (0.2320).^2;
r33     = (.0191).^2;

Q   = [q11  0   0       0;
       0    q22 0       0;
       0    0   q33     0;
       0    0   0       q44];
R   = [r11  0   0;
       0    r22 0;
       0    0   r33];

% Initiate E, S
E   = eye(2);
S   = zeros(2,1);

% Sampling interval
dt  = 1/Fs;

% Initiate state dynamics
A   = [1 dt dt^2/2 0; 0 1 dt 0; 0 0 1 1; 0 0 0 1];
B   = 0;
C   = eye(3);
D   = 0;

% Initiate P, K, H (=Cp)
P   = Q;
K  	= ones(4);
H   = [1 0 0 0; 0 1 0 0; 0 0 1 1];

% Run optimisation 
for j = 1 : numel(gap_idxs) - 1
    
    % Extract GPS measurements
    idxs        = gap_idxs(j) + 1 : gap_idxs(j + 1);
    lon         = GPS.longitude(idxs);
    lat         = GPS.latitude(idxs);
    
    % Extract speedometer, GPS and accelerometer measurements
    v_can       = CAN.vehicle_speed(idxs);
    v_gps       = GPS.speed(idxs);
    acc         = ACL.x(idxs);
    v_acl       = cumtrapz(dt,acc) + v_can(1);
    
    % Compute driven distance
    dist_GPS_v  = cumtrapz(dt,v_gps);
    dist_can    = cumtrapz(dt,v_can);
    
    dist_LL     = zeros(numel(idxs),1);    
    for i = 2 : numel(idxs)
        % Distance via lon/lat
        dist_LL(i)	= dist_LL(i-1) + ...
            lldistkm( ...
                [lat(i-1), lon(i-1)], ...
                [lat(i), lon(i)] ...
                ) * 1000;
    end
    
    % Initiate state estimation
    x_hat     	= zeros(4,numel(idxs));
    x_hat(:,1)	= [dist_GPS_v(1); v_can(1); acc(1); 0];
    
    % Perform actual optimisation ...
    for i = 1 : numel(idxs)
        x_hat_time      = A*x_hat(:,i);
        P_time          = A*P*A' + Q;
        
        K               = P_time * H'/(H*P_time*H' + R);
        y               = C*[dist_GPS_v(i); v_can(i); acc(i)];

        x_hat(:,i+1)    = x_hat_time + K*(y - H*x_hat_time);
        P               = (eye(4) - K*H)*P_time;
    end
        
    % Write to final matrix
    GPS.dist_LL(idxs)   = dist_LL;
    GPS.dist_GPS_v(idxs)= dist_GPS_v;
    GPS.dist_can(idxs)  = dist_can;
    GPS.speed_acl(idxs) = v_acl;
    GPS.dist_kal(idxs)  = x_hat(1,1:end-1)';
    GPS.speed_kal(idxs) = movmean(x_hat(2,1:end-1)',[filterWidth_ego_v filterWidth_ego_v]);
    GPS.acc_kal(idxs)   = x_hat(3,1:end-1)';
    GPS.v_a_kal(idxs)   = cumtrapz(dt,GPS.acc_kal(idxs)) + v_can(1);
    GPS.a_bias(idxs)    = x_hat(4,1:end-1)';
end

%% Plot figure: velocity improvement

hFig2 = figure();
set(hFig2, 'Position', [100 100 figWidth figHeight])
set(gcf,'Color','white');

% Add plots to first subplot
sp1 = subplot(2,1,1);
grid minor; hold on;
plot(GPS.Time,GPS.speed,'LineWidth',1.5)
plot(GPS.Time,GPS.speed_kal,'LineWidth',1.5)
plot(CAN.Time,CAN.vehicle_speed,'LineWidth',1.5)
plot(CAN.Time,GPS.speed_acl,'LineWidth',1.5)
plot(CAN.Time,GPS.v_a_kal,'LineWidth',1.5)
plot(GPS.Time,GPS.v_lim .* (1 - GPS.invalid),'LineWidth',1.5)

% Set subplot properties
legend('v_{GPS}','v_{CAN} (corrected, Kalman)',...
    'v_{CAN} (corrected, LS)','v_{ACL}','v_{a} (corrected, Kalman)',...
    'Speed limit',...
    'Orientation','Horizontal','Location','North')
xlabel('Time','FontSize',sFont,'FontWeight','normal');
ylabel('Speed, m/s','FontSize',sFont,'FontWeight','normal');
set(gca,'FontSize',sFont)
ylim([0 Inf])

% Compute driven distance via measurement signals
s0          = GPS.dist_LL(1);
x_ls_acl    = s0 + cumtrapz(dt,CAN.vehicle_speed) + ...
    cumtrapz(dt,cumtrapz(dt,ACL.x));

% Add plots to second subplot
sp2 = subplot(2,1,2);
grid minor; hold on;
plot(GPS.Time,abs((GPS.dist_kal - GPS.dist_LL) ./ GPS.dist_kal),'LineWidth',1.5)
plot(GPS.Time,abs((GPS.dist_kal - GPS.dist_can) ./ GPS.dist_kal),'LineWidth',1.5)

% Set subplot properties
legend('x_{lon,lat}','x_{Least Squares}',...
    'Orientation','Horizontal','Location','North')
title('Relative error lon-lat, least-squares with Kalman approach')
xlabel('Time','FontSize',sFont,'FontWeight','normal'); 
ylabel('Relative error','FontSize',sFont,'FontWeight','normal');
set(gca,'FontSize',sFont)
set(gca,'Yscale','log')

linkaxes([sp1,sp2],'x')

%% Plot figure: GPS sampling error

hFig = figure();
set(hFig, 'Position', [100 100 figWidth figHeight])
set(gcf,'Color','white');

% Add plots to figure
grid minor; hold on;
plot(GPS.Time,GPS.dist_LL,'LineWidth',1.5)
plot(GPS.Time,GPS.dist_can,'LineWidth',1.5)

% Set figure properties
legend('s_{LL}','s_{odo}',...
    'Orientation','Horizontal','Location','North')
title('Calculated driven distance from GPS coordinates')
xlabel('Time','FontSize',sFont,'FontWeight','normal');
ylabel('Distance, m','FontSize',sFont,'FontWeight','normal');
set(gca,'FontSize',sFont)
xlim([datetime([2018 06 13 08 16 27]),datetime([2018 06 13 08 16 48])]);

%% Plot figure: difference between odometer and lon/lat derived displacements

% Select field
fld_idxs = find(ismember(GPS.Properties.VariableNames,{'dist_can'}));

hFig3 = figure();
set(hFig3, 'Position', [100 100 figWidth figHeight])
set(gcf,'Color','white');

for k = 1 : 2
    if k == 1
        tmp_dist    = GPS.dist_LL;
        tmp_title   = 'Gap error over the velocity (GPS_{lonlat} / odometer)';
    else
        tmp_dist    = GPS.dist_GPS_v;
        tmp_title   = 'Gap error over the velocity (GPS_{speed} / odometer)';
    end
    
    for i = fld_idxs
        % Compute gap error
        v_vec           = linspace(0,max(GPS.speed),250);
        gap_err_by_v    = zeros(numel(v_vec)-1,1);
        gap_err_stdev   = zeros(numel(v_vec)-1,1);

        gap_err         = abs((diff(tmp_dist) - diff(GPS.dist_can)) ./ ...
                    diff(GPS.dist_can));

        tmp_gap_err_cumsum  = gap_err;
        tmp_gap_err_cumsum(abs(tmp_gap_err_cumsum )== Inf) = 0;
        tmp_gap_err_cumsum(isnan(tmp_gap_err_cumsum)) = 0;

        tmp_gap_err_mean    = gap_err;
        tmp_gap_err_mean(abs(tmp_gap_err_mean) == Inf) = NaN;

        num_in              = zeros(numel(v_vec)-1,1);

        for j = 1 : numel(v_vec) - 1
            gap_err_by_v(j) = mean(tmp_gap_err_mean(...
                GPS.speed(1:end-1) >= v_vec(j) & ...
                GPS.speed(1:end-1) < v_vec(j+1)));
            gap_err_stdev(j)= std(tmp_gap_err_mean(...
                GPS.speed(1:end-1) >= v_vec(j) & ...
                GPS.speed(1:end-1) < v_vec(j+1)));
            num_in(j)          = sum(GPS.speed(1:end-1) >= v_vec(j) & ...
                GPS.speed(1:end-1) < v_vec(j+1));
        end

        % Add plots to subplot
        sp1 = subplot(3,2,[k,k+2]);
        grid minor; hold on;
        confplot(v_vec(1:end-1),gap_err_by_v,2*gap_err_stdev,2*gap_err_stdev,'LineWidth',1.5)
        
        % Set subplot properties
        ylabel('Deviation, m','FontSize',sFont,'FontWeight','normal');
        title(tmp_title)
        xlabel('Velocity, m/s','FontSize',sFont,'FontWeight','normal');
        set(gca,'FontSize',sFont)
        legend('95% interval','','Mean')
        ylim([-1.5 1.5])
        xlim([0 40])

    end
end

% Add plots to subplot
sp2 = subplot(3,2,5:6);
grid minor; hold on;
plot(v_vec(1:end-1),num_in,'LineWidth',1.5)

% Set subplot properties
ylim([0 round(1.1*max(num_in(round(0.1*numel(num_in)):end)))])
ylabel('Number of datapoints','FontSize',sFont,'FontWeight','normal');
xlabel('Velocity, m/s','FontSize',sFont,'FontWeight','normal');
set(gca,'FontSize',sFont)
xlim([0 40])

%% Plot figure: accelerations resulting from different measurements

% Compute accelerations from all measurement signals
a_GPS_LL            = zeros(size(GPS.dist_LL,1),1);
a_v_kal             = zeros(size(GPS.speed_kal,1),1);
a_x_kal             = zeros(size(GPS.dist_kal,1),1);
for i = 2 : size(GPS.dist_LL,1) - 1
    a_GPS_LL(i)     = (GPS.dist_LL(i+1) - 2*GPS.dist_LL(i) + ...
        GPS.dist_LL(i-1)) ./ (dt^2);
    a_v_kal(i)      = (GPS.speed_kal(i+1) - GPS.speed_kal(i-1)) ./ (2 * dt);
    a_x_kal(i)      = (GPS.dist_kal(i+1) - 2*GPS.dist_kal(i) + ...
        GPS.dist_kal(i-1)) ./ (dt^2);
end
a_GPS_LL([1, end])  = a_GPS_LL([2, end-1]);
a_v_kal([1, end])   = a_v_kal([2, end-1]);
a_x_kal([1, end])   = a_x_kal([2, end-1]);

hFig3 = figure();
set(hFig3, 'Position', [100 100 figWidth figHeight])
set(gcf,'Color','white');
grid minor; hold on;

% Add plots to figure
plot(GPS.Time,a_v_kal,'LineWidth',1.5)
plot(GPS.Time,a_x_kal,'LineWidth',1.5)
plot(GPS.Time,GPS.acc_kal,'LineWidth',1.5)
plot(ACL.Time,ACL.x,'LineWidth',1.5,'LineWidth',1.5)

% Set figure properties
ylabel('Acceleration, m/s^2','FontSize',sFont,'FontWeight','normal');
xlabel('Time','FontSize',sFont,'FontWeight','normal');
title('Accelerations resulting from different sources')
set(gca,'FontSize',sFont)
ylim([-5 5])
legend('v_{Kalman}','s_{Kalman}','a_{Kalman}','ACL')

%% Plot figure: velocities resulting from different measurements

% Compute velocities
v_GPS_LL    = zeros(size(GPS.dist_LL,1),1);
v_x_kal     = zeros(size(GPS.dist_LL,1),1);
for i = 2 : size(GPS.dist_LL,1) - 1
    v_GPS_LL(i)     = (GPS.dist_LL(i+1) - GPS.dist_LL(i-1)) ./ (2 * dt);
    v_x_kal(i)      = (GPS.dist_kal(i+1) - GPS.dist_kal(i-1)) ./ (2 * dt);
end
v_GPS_LL([1, end])  = v_GPS_LL([2, end-1]);
v_x_kal([1, end])   = v_x_kal([2, end-1]);

v_GPS_LL(v_GPS_LL < 0)  = NaN;
v_x_kal(v_x_kal < 0)    = NaN;

hFig3 = figure();
set(hFig3, 'Position', [100 100 figWidth_panel .7*figHeight_panel])
set(gcf,'Color','white');

% Set figure properties
p1             = panel();
p1.pack(1,2)
p1.fontsize     = sFont;
p1.fontweight   = 'normal';
p1.margin = [12 6 2 3];      % Outside
p1.de.margin = 13;            % Internal

% Add plots to first subplot
p1(1,1).select();
grid minor; hold on;
plot(GPS.Time(1:end-1),GPS.speed_kal(2:end),'LineWidth',1.5)
plot(GPS.Time,GPS.v_a_kal,'LineWidth',1.5)
plot(GPS.Time,GPS.speed_acl,'LineWidth',1.5)

% Set subplot properties
ylabel('Velocity, m/s','FontSize',sFont,'FontWeight','normal');
set(gca,'FontSize',sFont)
legend('$$v_{Kal}$$','$$v_{Kal,a}$$','$$v_{ACL}$$', ...
    'Interpreter','Latex','Location','SouthEast')
xlim([datetime([2018 06 13 08 16 28]),...
    datetime([2018 06 13 08 17 35])]);
datetick('x','HH:MM:ss','keeplimits','keepticks')

% Add plots to second subplot
p1(1,2).select();
grid minor; hold on;
plot(GPS.Time(1:end-1),GPS.speed_kal(2:end),'LineWidth',1.5)
plot(GPS.Time,GPS.v_a_kal,'LineWidth',1.5)
plot(CAN.Time,CAN.vehicle_speed,'LineWidth',1.5)
plot(CAN.Time,GPS.speed,'LineWidth',1.5)

% Set subplot properties
ylabel('Velocity, m/s','FontSize',sFont,'FontWeight','normal');
set(gca,'FontSize',sFont)
legend('$$v_{Kal}$$','$$v_{Kal,a}$$','$$v_{CAN}$$','$$v_{GPS}$$', ...
    'Interpreter','Latex','Location','SouthEast','Orientation','Vertical')
xlim([datetime([2018 06 13 13 51 19]),...
    datetime([2018 06 13 13 51 27])]);
datetick('x','HH:MM:ss','keeplimits','keepticks')

%% Write final structure ...

GPS_kalman = GPS;
fprintf('Finished.\n')








