%% Program is written to compare logged accelerations and velocities

% Script written by:    Mathieu Blauw
% Date:                 
% Institution:          Delft University of Technology

%% Clear variables and add functions to current path
clearvars -except tab_all
addpath('./00_function_files_exchange/progressbar')                     % https://nl.mathworks.com/matlabcentral/fileexchange/6922-progressbar
addpath('./00_function_files_exchange/panel')                           % https://nl.mathworks.com/matlabcentral/fileexchange/20003-panel
addpath('./00_function_files_pre_analysis')
addpath('./00_function_files_synthetic_real')

%% Load data if not already existing ... 
if ~exist('tab_all','Var')
    tic
    load(fullfile('E:\Graduation\SAE-L2\Matlab','data_audi.mat'))
    t_load  = toc;
    fprintf(char(strcat({'Loaded data in '},num2str(t_load),{' seconds.\n'})))
else
    fprintf('Data already loaded. Continue without loading ...\n')
end
if ~exist('delta_k','Var')
    load(fullfile('../a2. output','delta_k_v2.mat'))
end
if ~exist('lags','Var')
    try
        load(fullfile('../a2. output','lags_with_correction_v3.mat'))
    end
end

%% Initiate ...

% ... figure properties
global sFont figWidth figHeight figWidth_panel figHeight_panel cmap
sFont       = 10;
figWidth    = 660;
figWidth_panel  = 600;
figHeight       = 250;
figHeight_panel = 225;
figure(100)
cmap        = colormap(lines(10));
close(figure(100))

% ... constants
g           = 9.80665;
t_start     = datetime([2018 06 13 00 00 00]);  % Start time [-]
t_end       = datetime([2018 06 14 00 00 00]);  % End time [-]
Fs          = 100;                              % Desired sampling frequency [s^-1]
t_range     = 300;                              % Considered time range [s]
idx_range   = t_range;                          % Considered index range [-]
t_thresh    = 20;                               % Minimum duration threshold for range [s]
idx_thresh  = t_thresh;                         % Minimum duration threshold for range [s]

%% Boolean options
print_figures               = true;
correct_sampling_interval 	= true;

%% Select data

% GPS data
GPS     = tab_all.DL004.Advantech_GPS.T_gps_raw;
GPS     = GPS(abs(GPS.longitude)<360,:);
GPS     = GPS(abs(GPS.longitude)<360,:);
GPS     = GPS(:,ismember(fieldnames(GPS),{'speed','longitude','latitude'}));
GPS.Properties.VariableContinuity = {'continuous','continuous','continuous'};
GPS(abs(GPS.speed) <= 0.1,:) = [];

% CAN-bus data
v_can   = tab_all.DL004.Advantech_VCIL_AUDI_A4.T_can_vehicle_speed_raw;
v_can   = v_can(:,ismember(fieldnames(v_can),{'vehicle_speed'}));
v_can(abs(v_can.vehicle_speed) <= 0.1,:)   = [];

% MobilEye data
ME   	= select_MobilEye(tab_all,t_start,t_end,Fs); 
ME  = ME(~isnan(ME.x),:);

%% Optionally: print figure showing delays (requires panel package)

if print_figures
    %%%%%% First figure (GPS delay) %%%%%%
    hFig = figure();
    set(hFig, 'Position', [100 100 figWidth_panel figHeight_panel])
    set(gcf,'Color','white');
    
    p1           	= panel();
    p1.pack(2,1)

    % Define window limits x-axis
    tmp_t1  = datetime([2018 06 13 16 14 44]);
    tmp_t2  = datetime([2018 06 13 16 14 56]);
    
    % Find changes lead vehicle ID
    d_ID            = diff(ME.ID);
    d_ID(d_ID ~= 0) = 1;
    
    % Add plots to first subplot
    hAx(1) = p1(1,1).select();
    hold on; grid minor
    plot(GPS.Time,GPS.speed,'--','LineWidth',.5,'Color',cmap(1,:))
    plot(GPS.Time + seconds(lags.t_lag_rmse)/100,GPS.speed,'LineWidth',1.5,'Color',cmap(1,:))
    plot(v_can.Time,v_can.vehicle_speed,'--','LineWidth',.5,'Color',cmap(2,:))
    plot(v_can.Time,v_can.vehicle_speed*(1-delta_k),'LineWidth',1.5,'Color',cmap(2,:))
    plot(v_can.Time(1:2),[NaN,NaN],'Color',cmap(3,:),'LineWidth',1.5)
    plot(v_can.Time, v_can.vehicle_speed*(1-delta_k),'*','LineWidth',1,'Color',cmap(4,:))
     
    % Set subplot properties
    ylabel('Velocity, m/s','FontSize',sFont,'FontWeight','normal');
    set(gca,'FontSize',sFont)
    set(gca,'XTickLabels',[]);
    
    % Add plots to first subplot
    hAx(2) = p1(2,1).select();
    hold on; grid minor
    plot(ME.Time, ME.x,'LineWidth',.5,'Color',cmap(3,:),'LineWidth',1.5)
    plot(ME.Time(find(d_ID==1)+1), ME.x(find(d_ID==1)+1),'*','LineWidth',1,'Color',cmap(4,:))
    
    % Set subplot properties
    ylabel('Distance-gap, m','FontSize',sFont,'FontWeight','normal');
    set(gca,'FontSize',sFont)

    % Link axes and set axes limits
    linkaxes(hAx,'x')
    set(hAx,'xlim',([tmp_t1 tmp_t2]))
    datetick('x','HH:MM:ss','keeplimits','keepticks')
    
    % Add legend
    p1(1,1).select();
    lgnd = legend('$$v_{GPS}$$','$$v_{GPS}$$ (corrected)', ...
    '$$v_{CAN}$$', '$$v_{CAN}$$ (corrected)','s','Reset','Interpreter','Latex',...
    'Orientation','Horizontal','Location','South');
    lgnd.ItemTokenSize(1) = 15;
    
    % Remove some stuff from legend ...
    ax = get(gcf,'children');
    ind = find(isgraphics(ax,'Legend'));
    set(gcf,'children',ax([ind:end,1:ind-1]))
    
    % Finalise figure
    p1.fontsize    	= sFont;
    p1.fontweight   = 'normal';
    p1.margin = [11 7 10 3];        % Outside
    p1.de.margin = 7;               % Internal

    %%
    %%%%%% Second figure (sampling time distribution) %%%%%%
    hFig2 = figure();
    set(hFig2, 'Position', [100 100 700 400])
    set(gcf,'Color','white');

    % Add to plot
    hold on; grid minor
    tmp_dt  = milliseconds(diff(GPS.Time));
    tmp_dt(tmp_dt > 5000)  = NaN;

    histogram(tmp_dt,'Normalization','Probability')

    % Set plot properties
    title('Distribution of sampling times for the GPS signal'); 
    xlabel('Time','FontSize',sFont,'FontWeight','normal');     
    ylabel('Probability, -','FontSize',sFont,'FontWeight','normal');
    set(gca,'FontSize',sFont)
    xlim([0 2500])

    %%
    %%%%%% Third figure (sampling time inconsistency) %%%%%%
    % Group sampling intervals
    dt_small = find(tmp_dt > 300 & tmp_dt < 700);
    dt_medium = find(tmp_dt > 1300 & tmp_dt < 1750);
    dt_large = find(tmp_dt > 1750 & tmp_dt < 2050);

    hFig3 = figure();
    set(hFig3, 'Position', [100 100 1400 300])
    set(gcf,'Color','white');

    % Add plots to figure
    hold on; grid minor
    plot(GPS.Time(dt_small),ones(numel(dt_small),1),'*')
    plot(GPS.Time(dt_medium),ones(numel(dt_medium),1),'*')
    plot(GPS.Time(dt_large),ones(numel(dt_large),1),'*')

    % Set figure properties
    legend('.4 - .6 s','1.4 - 1.6 s','1.9 - 2.05 s',...
        'Orientation','Horizontal','Location','North')        
    title('Patterns of sampling intervals over time'); 
    xlabel('Time','FontSize',sFont,'FontWeight','normal');     
    ylabel('Sampling intervals, -','FontSize',sFont,'FontWeight','normal');
    set(gca,'FontSize',sFont)
    
    % Clear temporary variables
    clear tmp*
end

%% Find GPS lag

execute_loop    = input('Execute loop? Yes(1)/No - load data instead(0)');

if execute_loop
    % Initiate correction vectors
    t_lag_rmse  = NaN(size(GPS,1),1);
    t_shift_err = zeros(size(GPS,1),1);
    
    if correct_sampling_interval
        %%%%%% Remove long-short sampling inconsistency %%%%%%
        for i_medium = dt_medium'
            % Find consecutive long-short samples
            i_small  = dt_small(dt_small == i_medium + 1);

            if numel(i_small) == 1
                % Save current time to vector
                tmp_shift_err       = GPS.Time(i_small);
                
                % Compute new time
                %   (n.b. new time is earlier than old time)
                GPS.Time(i_small)   = datetime( ...
                    (datenum(GPS.Time(i_small+1)) + ...
                    datenum(GPS.Time(i_small-1))) / 2, ...
                    'ConvertFrom','datenum');
                
                % Compute difference 
                %   (= old time - new time, positive means shift to left)
                t_shift_err(i_small) = (milliseconds(tmp_shift_err - ...
                    GPS.Time(i_small))) / 10;
            end
        end
        
        % Shift sign of t_shift_err to match lag vector
        t_shift_err     = - t_shift_err;
    end

    % Find start and end indices for required interval
    [~,idx_t_start] = min(abs(GPS.Time - t_start));
    [~,idx_t_end]   = min(abs(GPS.Time - t_end));

    % Find missing data
    tmp_idx_missing = find(((GPS.Time(idx_t_start+1:idx_t_end) - ...
        GPS.Time(idx_t_start:idx_t_end-1)) >= seconds(t_thresh))) + (idx_t_start-1);
    idx_missing     = [idx_t_start; tmp_idx_missing; idx_t_end];
    
    % Initiate progressbar
    progressbar(0,0)
    for j = 1 : numel(idx_missing) - 1    
    % Find first and last index within current interval ...
    idx_start_GPS   = idx_missing(j) + 1;
    idx_end_GPS     = idx_missing(j + 1);
    
    k = 0;
    % ... and loop through those indexes
        for i = idx_start_GPS : idx_end_GPS
            k = k + 1;
            
            % Update progressbar once per 10 iterations
            if k == 10
                progressbar((j-1)/numel(idx_missing),(i-idx_start_GPS-1)/numel(idx_start_GPS : idx_end_GPS))
                k = 0;
            end
            % Find the range boundaries for the current index
            interval_start_idx  = max(i - idx_range/2, idx_start_GPS);
            interval_end_idx    = min(i + idx_range/2, idx_end_GPS);
            % Only compute the lag if interval is long enough
            if (interval_end_idx - interval_start_idx) >= idx_thresh
                % Only keep those indexes within the range
                GPS_tmp     = GPS(interval_start_idx:interval_end_idx,:);

                % ... and also do that for the CAN-bus signal
                v_can_tmp   = v_can(v_can.Time >= GPS_tmp.Time(1),:);
                v_can_tmp   = v_can_tmp(v_can_tmp.Time <= GPS_tmp.Time(end),:);

                % Resample signals
                GPS_tmp     = retime(GPS_tmp,'regular','linear','SampleRate',Fs);
                v_can_tmp   = retime(v_can_tmp,'regular','linear','SampleRate',Fs);

                % Filter the CAN-bus signal
                v_can_tmp_filtered          = movmean(v_can_tmp.vehicle_speed,Fs);
                v_can_tmp.vehicle_speed     = v_can_tmp_filtered;

                % Only keep common indices
                [~,ia,ib]   = intersect(GPS_tmp.Time,v_can_tmp.Time);
                GPS_tmp     = GPS_tmp(ia,:);
                v_can_tmp   = v_can_tmp(ib,:);

                if numel(v_can_tmp) > 0 && numel(GPS_tmp) > 0
                    % (Removed) Match via cross-correlation (explained variance)
                    %   Cross-correlation is used to find the lag for which maximum
                    %   correlation coefficient is found
                    
                    % Match via RMSE (unexplained variance)
                    t_lag_rmse(i)    = compute_lag_RMSE(GPS_tmp.speed,v_can_tmp.vehicle_speed,Fs);
                end
            else
                % Set to NaN if interval length is too short ...
                t_lag_rmse(i)   = NaN;
            end
        end
    end
    progressbar(1)
    
    % Set those indices to mean value
    t_lag_rmse(isnan(t_lag_rmse)) = mean(t_lag_rmse(~isnan(t_lag_rmse)));
    t_lag_rmse      = t_lag_rmse + t_shift_err;
    GPS.t_lag_rmse  = t_lag_rmse;
    lags            = GPS(:,ismember(fieldnames(GPS),{'t_lag_rmse'}));

    fprintf('\n Process complete, do not forget to save lags.mat\n')
else % If loading required, do so ...
    load('./00_output/lags_with_correction')
    GPS.t_lag_rmse  = lags.t_lag_rmse;
end

%% Print figure showing lag over time 
try
    close(hFig)
end

hFig = figure();
set(hFig, 'Position', [100 100 1400 800])
set(gcf,'Color','white');

% Add plots to first subplot
sp1 = subplot(2,1,1);
hold on
plot(GPS.Time, GPS.speed,'LineWidth',1.5)
plot(GPS.Time + seconds(GPS.t_lag_rmse/Fs), GPS.speed,'LineWidth',1.5)
plot(GPS.Time + seconds(-0.8), GPS.speed,'LineWidth',1.5)
plot(v_can.Time, v_can.vehicle_speed*(1-delta_k),'LineWidth',1.5)

% Set subplot properties
legend('GPS (original)','GPS (RMSE)','GPS (-.8s)','CAN (adjusted)')
set(gca,'YLim',[0 41])
ylabel('Speed, m/s','FontSize',sFont,'FontWeight','normal');     
title('Velocity profiles'); 

% Add plots to first subplot
sp2 = subplot(2,1,2);
hold on
plot(GPS.Time,movmean(GPS.t_lag_rmse,1),'LineWidth',1.5)

% Set subplot properties
axis tight
legend('Lag (RMSE)')
set(gca,'YLim',[-150 0])
ylabel('Lag, ms','FontSize',sFont,'FontWeight','normal');     
title('Lag profile'); 

for i = 1 : 2
    subplot(2,1,i)
    grid minor
    xlabel('Time','FontSize',sFont,'FontWeight','normal');     
    set(gca,'FontSize',sFont)
end

linkaxes([sp1,sp2],'x')
ax          = gca;

%% Print figure showing lag difference over time 
try
    close(hFig2)
end

hFig2 = figure();
set(hFig2, 'Position', [100 100 700 400])
set(gcf,'Color','white');

% Add plots to figure
sp1 = subplot(1,1,1);
hold on
histogram(diff(GPS.t_lag_rmse),'Normalization','probability','BinWidth',1)

% Set figure properties
axis tight
set(gca,'XLim',[-10,10])
ylabel('Probability, -','FontSize',sFont,'FontWeight','normal'); 
xlabel('Difference, ms','FontSize',sFont,'FontWeight','normal');

for i = 1 : 1
    subplot(1,1,i)
    grid minor
    set(gca,'FontSize',sFont)
    title('Difference in calculated lag between two consecutive points'); 
end


