%% This script is written to aggregate the ACC data from different sources

% Script written by:    Mathieu Blauw
% Date:                 April 5th, 2019
% Institution:          Delft University of Technology

%% Clear variables and add functions to current path
clearvars -except tab_all GPS_lag delta_k
addpath('./00_function_files_exchange/progressbar')                      % https://nl.mathworks.com/matlabcentral/fileexchange/6922-progressbar
addpath('./00_function_files_exchange/lonlat_to_km')                     % https://nl.mathworks.com/matlabcentral/fileexchange/38812-latlon-distance
addpath('./00_function_files_exchange/panel')                            % https://nl.mathworks.com/matlabcentral/fileexchange/20003-panel
addpath('./00_function_files_exchange/export_fig')                       % https://nl.mathworks.com/matlabcentral/fileexchange/23629-export_fig
    % export_fig('test_image.pdf', '-pdf', '-c[NaN,NaN,NaN,NaN]', '-transparent', '-CMYK')
addpath('./00_function_files_synthetic_real')
addpath('./00_function_files_pre_analysis')

%% Load data if not already existing ... 
tic
load(fullfile('E:\Graduation\SAE-L2\Matlab','data_audi.mat'))
t_load  = toc;
fprintf(char(strcat({'Loaded data in '},num2str(t_load),{' seconds.\n'})))

if ~exist('GPS_lag','Var')
    load(fullfile('../a2. output','lags_with_correction_v3.mat'))
end
if ~exist('delta_k','Var')
    load(fullfile('../a2. output','delta_k_v2.mat'))
end
if ~exist('GPS_kalman','Var')
    load(fullfile('../a2. output','output_kalman_v6.mat'))
end

%% Initiate ...

% ... figure properties
global sFont figWidth figHeight figWidth_panel figHeight_panel cmap
sFont       = 10;
figWidth    = 660;
figWidth_panel  = 600;
figHeight       = 250;
figHeight_panel = 225;

figure(100)
cmap    = colormap(lines(100));
close(figure(100))

% ... constants
g       = 9.80665;
t_start = datetime([2018 06 13 00 00 00]);
t_end   = datetime([2018 06 14 00 00 00]);
Fs      = 10;
dt      = 1/Fs;

%% Select data
if exist('CAN','var')
    reload_bool     = input('Reload data? Yes(1)/No(0)');
else
    reload_bool     = 1;
end
if reload_bool
    ME                  = select_MobilEye(tab_all,t_start,t_end,Fs);    %
    CAN                 = select_CAN(tab_all,t_start,t_end,Fs);         %
    CAN.vehicle_speed   = CAN.vehicle_speed * (1-delta_k);              % Correct CAN-data bias
%     GPS                 = select_GPS(tab_all,lags,t_start,t_end,Fs);    %
    ACL                 = select_ACL(tab_all,t_start,t_end,Fs);         %
end

%% ACC state detection
% Effective duration of effects ACC state change
%       effective_duration = [seconds before, seconds after deactivation]
effective_duration  = [-1 5];

% Find suitable times for ACC (de)activation
suitable_times_ACC  = find_suitable_times_ACC(CAN,effective_duration);

%% Lane change detection
% Minimum peak height [m], maximum time between peaks [s],
% effective duration of effects lane-change
%       effective_duration = [seconds before/after];
d_threshold         = 2;        % [m], minimum peak height (equals width of new lane)
t_threshold         = 1;        % [s], maximum time between logged lane changes
effective_duration  = 5;        % [s], duration of the lane-change effects

% Load MobilEye lane information
d_marks             = tab_all.DL004.Advantech_MobilEye.T_mobileye_0x669_raw;

% Find suitable times for lane changing
suitable_times_LC   = find_suitable_times_LC(CAN,ME,d_marks,effective_duration,...
    t_threshold,d_threshold);

%% MobilEye detection
% Detect times at which the MobilEye system has correctly identified the
% vehicle driving in front (cipv = 1, lane = 1 (ego-lane))
ego_lane            = (ME.lane == 1);
lane_cipv           = ME.cipv .* ego_lane;
ME(~lane_cipv,:)    = [];

%% Equalize length of all tables
[~,i_ACL,i_CAN,i_ME,i_ACC,i_LC,i_kal] = ...
    intersectm(ACL.Time,CAN.Time,ME.Time, ...
    suitable_times_ACC.Time,suitable_times_LC.Time,...
    GPS_kalman.Time);

ACL                 = ACL(i_ACL,:);
CAN                 = CAN(i_CAN,:);
ME                  = ME(i_ME,:);
suitable_times_ACC 	= suitable_times_ACC(i_ACC,:);
suitable_times_LC	= suitable_times_LC(i_LC,:);
GPS_kalman          = GPS_kalman(i_kal,:);
clear i_*

%% Driving regimes detection

s_c         = 25;       % [m]       Gap threshold (standstill detection)
v_c         = 30/3.6;   % [m/s]     Velocity threshold
a_c         = 1.0;      % [m/s2]    Acceleration threshold
a_c_kin     = 0.1;      % [m/s2]    Kinematic deceleration threshold
T_c         = 2.5;      % [s]       Gap threshold

% Already construct part of the final table ...
AUDI        = detect_driving_regimes(dt, GPS_kalman, CAN, ME, ...
    s_c, v_c, a_c, a_c_kin, T_c);

%% Construct final table

% t_start     = datetime([2018 06 13 06 30 00]);
% t_end       = datetime([2018 06 13 08 00 00]);

% Finalize construction of the final table
[final_table, times] = construct_final_table( ...
    suitable_times_ACC, suitable_times_LC, ...
    AUDI, t_start, t_end);

% Correct missing gear values
final_table.gear(cellfun('isempty', final_table.gear))  = {'D'};

AUDI        = final_table;

fprintf('Finished.\n');



















