%% This script is written to investigate the performance of the cailbration
%   procedure to synthetic data

% Script written by:    Mathieu Blauw
% Date:                 May 23, 2019
% Institution:          Delft University of Technology

%% Main conclusions

%

%%
clearvars 
addpath('./00_function_files_exchange/progressbar')                     % https://nl.mathworks.com/matlabcentral/fileexchange/6922-progressbar
addpath('./00_function_files_exchange/symlog')                          % https://nl.mathworks.com/matlabcentral/fileexchange/57902-symlog
addpath('./00_function_files_exchange/panel')                           % https://nl.mathworks.com/matlabcentral/fileexchange/20003-panel
addpath('./00_function_files_exchange/permn')                           % https://nl.mathworks.com/matlabcentral/fileexchange/7147-permn
addpath('./00_function_files_exchange/export_fig')                      % https://nl.mathworks.com/matlabcentral/fileexchange/23629-export_fig
    % export_fig('test_image.pdf', '-pdf', '-c[NaN,NaN,NaN,NaN]', '-transparent', '-CMYK')
addpath('./00_function_files_synthetic_real')
addpath('./06_function_files_synthetic')
addpath('./00_function_files_CF_models')

%% Load data if not already existing ... 
tic
load(fullfile('../a2. output','final_table_v11.mat'))
t_load  = toc;
fprintf(char(strcat({'Loaded data in '},num2str(round(t_load,1)),{' seconds.\n'})))

%% Initiate some stuff ...

% General
% Make sure all random sequences are repeatable
rng(0,'twister');

% Plot settings
global sFont figWidth figHeight figWidth_panel figHeight_panel cmap
sFont       = 10;
figWidth    = 660;
figWidth_panel  = 600;
figHeight       = 250;
figHeight_panel = 225;
figure(100)
cmap        = colormap(lines(10));
close(figure(100))

% Data settings
global Fs_data dt_data t_int_min
global calibration_setup

Fs_data         = 10;                       % [Hz], Frequency dataset
dt_data         = 1/Fs_data;                % [s], Time-step size dataset
Fs_sim          = 10;                       % [Hz], Frequency simulation
calibration_setup.dt_sim    = 1/Fs_sim;     % [s], Time-step size simulation
t_int_min       = .11;                      % [s], Minimum span of time-interval
N_points        = 100;                      % Number of evaluations for landscape plot

% Calibration settings
%   -> General
use_parallel        = true;
max_generations     = 1000;
func_tolerance      = eps;
crossover_frac      = .8;
max_time            = 300;

% Options: https://nl.mathworks.com/help/gads/genetic-algorithm-options.html
global options

options             = optimoptions('fmincon','Algorithm','sqp', ...
    'Display','off', 'OptimalityTolerance', eps, 'FunctionTolerance', eps, ...
    'StepTolerance', eps);

% Calibration settings
%   -> Error measure settings if no variations are applied
calibration_setup.include_gap_err   = 1;
calibration_setup.include_speed_err = 1;
calibration_setup.error_measure     = 'RMSE';     	% SSE / RMSE/RMSRE / MAE/MARE
calibration_setup.frac_gap          = .5;
calibration_setup.frac_speed        = 1 - calibration_setup.frac_gap;
calibration_setup.calibration_variable  = 's';
calibration_setup.base_variable     = 'v';          % Base-variable for ego-vehicle ('v'/'x')
calibration_setup.noise             = '0-0.1';   	% noise mean-variance
calibration_setup.noise_type        = 'none';     	% None (none), leader (L), or both (FL)
calibration_setup.t_reset           = 5000;
calibration_setup.data_length       = Inf;

% Pre-set parameter values
global p_ACC_pub p_IDM_pub
global kp_range kd_range td_range
global a_range b_range T_range v0_range s0_range

p_ACC_pub           = [.23 .07 1.4];
p_IDM_pub           = [1.25 2.09 1.4 55 3];

%% Print calibration settings

fprintf(char(strcat( ...
    '\n ########################\n   Calibration summary \n\n', ...
    'Error measure \t\t\t %s \n ', ...
    'Calibration Variable \t %s \n ', ...    
    'Include gap error \t \t %d \nInclude speed error \t %d \n ', ...
    'Fraction gap \t\t\t %g \nFraction speed \t\t\t %g \n ', ...    
    ' ########################\n' ...
    )), ...
    calibration_setup.error_measure, ...
    calibration_setup.calibration_variable, ...
    calibration_setup.include_gap_err, ...
    calibration_setup.include_speed_err, ...
    calibration_setup.frac_gap, ...
    calibration_setup.frac_speed)

pause(2)

%% Select data
% Selected subsets of data
%       Subset      v_range     a_range     remarks
%       ---------   ---------   ---------   ---------
%       CF          Highway     [-1, 1]     car-following, low acc-/dec
%       OSC         Highway     [-2, 2]     oscillatory, moderate acc-/dec
%       HAC         Highway     [-4, 4]     High acc-/dec
%       SG1         Full        [-4, 2]     Full stop-and-go, high dec, moderate acc
%       SG2         Full        [-4, 4]     Full stop-and-go, high acc-/dec

data_CF         = AUDI(AUDI.Time > datetime([2018 06 13 06 48 23]) & ...
    AUDI.Time < datetime([2018 06 13 06 54 54]),:);

data_OSC        = AUDI(AUDI.Time > datetime([2018 06 13 10 54 30]) & ...
    AUDI.Time < datetime([2018 06 13 10 57 31]),:);
data_OSC        = [data_CF(1:end-size(data_OSC,1),:); data_OSC];

data_HAC        = AUDI(AUDI.Time > datetime([2018 06 13 14 16 00]) & ...
    AUDI.Time < datetime([2018 06 13 14 18 01]),:);
data_HAC        = [data_CF(1:end-size(data_HAC,1),:); data_HAC];

data_SG1        = AUDI(AUDI.Time > datetime([2018 06 13 14 16 30]) & ...
    AUDI.Time < datetime([2018 06 13 14 23 01]),:);

data_SG2        = AUDI(AUDI.Time > datetime([2018 06 13 14 42 21]) & ...
    AUDI.Time < datetime([2018 06 13 14 49 01]),:);
data_SG2        = data_SG2(1:end-3,:);

data_all.CF     = data_CF;
data_all.OSC    = data_OSC;
% data_all.HAC    = data_HAC;
data_all.SG1    = data_SG1;
data_all.SG2    = data_SG2;

%% Run variations
FN_dataset          = fieldnames(data_all);
FN_noise            = {'L','FL','none'};
FN_noiselvls        = {'0-0.1','0-0.05','0-0.01','0-0.001'};
FN_calivar          = {'s','sv','v'};
FN_calivar_label    = {'100-0','50-50','0-100'};
error_meas_vec      = {'RMSE','RMSRE','MAE','MARE'};
FN_reset            = {'5000','50','25','10','1'};

% Loop through all data
for i = 1 : numel(FN_dataset)
    % Select current data
    data            = data_all.(FN_dataset{i});
    
    % Loop through all reset interval possibilities
    % {'5000','50','25','10','1'}
    for s = 1 : numel(FN_reset)
        % Write current reset interval to calibration_setup
        calibration_setup.t_reset   = FN_reset{s};

        %% Construct reset indices and calculate dependent variables

        % Find reset indices and remove too short intervals
        data_w_reset    = find_reset_points(data);

        % Calculate dependent variables
        data_w_vars     = get_dependent_vars(data_w_reset, ...
            calibration_setup.base_variable);

        %% Create synthetic data from dataset (ACC)
        Np              = 6;                % Number of variations per parameter
        
        kp_range        = [.2 .3];             
        kd_range        = [.01 .1];
        td_range        = [1 3.6];

        [p_ACC_syn, data_syn_ACC, ~] = ...
            create_synthetic_data_ACC(data_w_vars, Np, false);

        % Create synthetic data from dataset (IDM)
        Np              = 3;                % Number of variations per parameter
        a_range         = [1 3];
        b_range         = [1 3];
        T_range         = [1 3.6];
        v0_range        = [45 55];
        s0_range        = [1 5];

        [p_IDM_syn, data_syn_IDM, ~] = ...
            create_synthetic_data_IDM(data_w_vars, Np, false);
        
        % Loop through all noise type possibilities
        %   {'L','FL','none'};
        for j = 1 : numel(FN_noise)
            % Write current noise to calibration_setup
            calibration_setup.noise_type    = FN_noise{j};

            % Loop through all noise level possibilities
            %   {'0-0.1','0-0.05','0-0.01','0-0.001'};
            for r = 1 : numel(FN_noiselvls)
                % Write current noise level to calibration_setup
                calibration_setup.noise     = FN_noiselvls{r};

                % Loop through all calibration variable possibilities
                %   (variable weight)
                %   {'s','sv','v'};
                for k = 1 : numel(FN_calivar)
                    % Write current calibration variable to calibration_setup
                    calibration_setup.calibration_variable    = FN_calivar{k};

                    % Loop through all error_measure possibilities
                    %   {'RMSE','RMSRE','MAE','MARE'};
                    for q = 1 : numel(error_meas_vec)
                        % Write current error measure to calibration_setup
                        calibration_setup.error_measure         = error_meas_vec{q};

                        %% Plot trajectories with follower trajectory

                        plot_all_trajectories(data_all, false)
                        % Only run loop if data is not yet present
                        fullFolder  = strcat('./loop_output/',FN_dataset{i},'/', ...
                            FN_noise{j},'/',FN_calivar{k},'/landscape');

                        %% Create fitness landscape of the dataset
                        close all

                        fprintf('Currently evaluating: %s, %s, %s, %s, %s, landscape\n', ...
                            FN_dataset{i},FN_noise{j},FN_noiselvls{r},FN_calivar{k},error_meas_vec{q})

                        % plot_fitness_landscape_XXX(follower_data, landscape_resolution, trajectory_generation_model)
                        %  Script automatically adds noise if required
                        err_IDM = plot_fitness_landscape_IDM(AUDI, data_w_vars, N_points, 'ACC');
                        err_ACC = plot_fitness_landscape_ACC(AUDI, data_w_vars, N_points, 'ACC');


                    end
                end
            end
            
            %% Add noise to the trajectories
            noise           = str2double(strsplit(calibration_setup.noise,'-'));
            noise_type      = calibration_setup.noise_type;
            noise_mean      = noise(1);
            noise_variance  = noise(2);

            % data_out = add_noise(data_in, noise mean, noise variance)
            data_syn_IDM_noisy      = add_noise_to_trajectory(data_syn_IDM, ...
                noise_type, noise_mean, noise_variance);
            data_syn_ACC_noisy      = add_noise_to_trajectory(data_syn_ACC, ...
                noise_type, noise_mean, noise_variance);

            %% Add variations to the synthetic data
            % Select type of variation to add
            %   none                    - Add no variations
            %   error_measure           - Selects the following measures: 'SSE','RMSE','RMSRE','MAE','MARE' 
            %   calibration_variable    - Selects the following variables: 's','sv', 'v'
            %   variable_weight         - Selects the following weights (s-v: 100-0, 90-10, 75-25, 50-50, 25-75, 10-90, 0-100)
            %   length                  - Selects the following lengths (in seconds): Inf 300 250 200 150 100
            %   noise_L/FL              - Selects the following noise levels: .1 .05 .02 .01 .001 (L - Leader, FL - both)
            %   reset                   - Selects the following reset time-intervals: n/a, 50, 25, 10, 1, .2, .1

            type_vec        = {'error_measure','noise_L','noise_FL', ...
                'reset','variable_weight','length','none'};

            for m = 1 : numel(type_vec)
                    type            = type_vec{m};
                    
                    fprintf('Currently evaluating: %s, %s, %s, %s\n', ...
                        FN_dataset{i},FN_noise{j},FN_calivar{k}, type)
                    
                    % Make sure not both 'noisy data' and 'noise variations' are selected
                    if (strcmpi(type,'noise_FL') || strcmpi(type,'noise_L')) && ...
                            ((noise_mean ~= 0 || noise_variance ~= 0) && ~strcmpi(noise_type,'none'))
                        continue
                    else
                        data_in_ACC = data_syn_ACC_noisy;
                        data_in_IDM = data_syn_IDM_noisy;
                    end

                    % Add variations to the dataset
                    [p_ACC_syn_w_variations, data_syn_ACC_w_variations, varinfo_ACC] = ...
                        add_variations_synthetic_data(p_ACC_syn, data_in_ACC, type);
                    [p_IDM_syn_w_variations, data_syn_IDM_w_variations, varinfo_IDM] = ...
                        add_variations_synthetic_data(p_IDM_syn, data_in_IDM, type);

                    % Create vector containing variation labels
                    labels_ACC  = varinfo_ACC.labels;
                    labels_IDM  = varinfo_IDM.labels;
                    
                    %% Calibrate models

                    % Calibrate models to generated lead trajectories
                    [p_ACC,cost_ACC,~,~]	= calibrate_ACC(data_syn_ACC_w_variations);
                    [p_IDM,cost_IDM,~,~]	= calibrate_IDM(data_syn_IDM_w_variations);

                    %%
                    p_names_ACC             = {'k_p, s^{-2}','k_d, s^{-1}','t_d, s'};
                    p_names_IDM             = {'a, m/s^2','b, m/s^2','T, s','v_0, m/s','s_0, m'};
                  
                    % Plot distribution calibrated parameters
                    [hFig, p] = plot_parameter_distribution(...
                        p_IDM, p_IDM_syn_w_variations, 'absrel', ...
                        p_names_IDM, labels_IDM, type, varinfo_IDM, 'IDM');
                    [hFig2, p2]   = plot_parameter_distribution(...
                        p_ACC, p_ACC_syn_w_variations, 'absrel', ...
                        p_names_ACC, labels_ACC, type, varinfo_ACC, 'ACC', hFig, p);

                    % Plot length variations
                    %   (Only in case variation == length)
                    plot_length_variation(data_syn_IDM, p_IDM, ...
                        p_IDM_syn_w_variations, 'absrel', p_names_IDM, varinfo_IDM, 'IDM', FN_dataset{ii});

                    plot_length_variation(data_syn_ACC, p_ACC, ...
                        p_ACC_syn_w_variations, 'absrel', p_names_ACC, varinfo_ACC, 'ACC', FN_dataset{ii});

            end
        end
    end
end
















