%% This script is written to investigate the performance on real-world vehicle data

% Script written by:    Mathieu Blauw
% Date:                 August 5th, 2019
% Institution:          Delft University of Technology

%% Main conclusions

%

%%
clearvars 
addpath('./00_function_files_exchange/progressbar')                      % https://nl.mathworks.com/matlabcentral/fileexchange/6922-progressbar
addpath('./00_function_files_exchange/panel')                            % https://nl.mathworks.com/matlabcentral/fileexchange/20003-panel
addpath('./00_function_files_exchange/align_YLabels')                    % https://nl.mathworks.com/matlabcentral/fileexchange/41701-y-labels-alignment-in-subplots
addpath('./00_function_files_exchange/permn')                            % https://nl.mathworks.com/matlabcentral/fileexchange/7147-permn
addpath('./00_function_files_exchange/export_fig')                       % https://nl.mathworks.com/matlabcentral/fileexchange/23629-export_fig
    % export_fig('test_image.pdf', '-pdf', '-c[NaN,NaN,NaN,NaN]', '-transparent', '-CMYK')
addpath('./00_function_files_synthetic_real')
addpath('./07_function_files_real_world')
addpath('./00_function_files_CF_models')

%% Load data if not already existing ... 
tic
load(fullfile('../a2. output','final_table_v11.mat'))
t_load  = toc;
fprintf(char(strcat({'Loaded data in '},num2str(round(t_load,1)),{' seconds.\n'})))

%% Constants
% Plot settings
global sFont figWidth figHeight figWidth_panel figHeight_panel cmap
sFont           = 10;
figWidth        = 660;
figWidth_panel  = 600;
figHeight       = 250;
figHeight_panel = 225;
figure(100)
cmap        = colormap(lines(10));
close(figure(100))

% Data settings
global Fs_data dt_data t_int_min
global calibration_setup

Fs_data         = 10;           % [Hz], Frequency dataset
dt_data         = 1/Fs_data;    % [s], Time-step size dataset
Fs_sim          = 10;           % [Hz], Frequency simulation
calibration_setup.dt_sim    = 1/Fs_sim; % [s], Time-step size simulation
t_int_min       = .11;           % [s], Minimum span of time-interval
N_points        = 100;           % Number of evaluations for landscape plot
N_points        = 5;           % Number of evaluations for landscape plot

% Calibration settings
%   -> General
use_parallel        = true;
max_generations     = 1000;
func_tolerance      = eps;
crossover_frac      = .8;
max_time            = 300;

% Options: https://nl.mathworks.com/help/gads/genetic-algorithm-options.html
global options

options             = optimoptions('fmincon','Algorithm','sqp','Display','final', ...
    'MaxFunctionEvaluations',1e5,'OptimalityTolerance', eps, 'FunctionTolerance', eps, ...
    'StepTolerance', eps);

% Calibration settings
%   -> Error measure settings if no variations are applied

calibration_setup.include_gap_err   = 1;
calibration_setup.include_speed_err = 1;
calibration_setup.error_measure     = 'RMSE';     	% SSE / RMSE/RMSRE / MAE/MARE
calibration_setup.frac_gap          = .5;
calibration_setup.frac_speed        = 1 - calibration_setup.frac_gap;
calibration_setup.calibration_variable  = 'sv';
calibration_setup.base_variable     = 'v';          % Base-variable for ego-vehicle ('v'/'x')
calibration_setup.noise             = '0-0.1';   	% noise mean-variance
calibration_setup.noise_type        = 'none';     	% None (none), leader (L), or both (FL)
calibration_setup.t_reset           = 25;
calibration_setup.data_length       = Inf;

% Pre-set parameter values
global p_ACC_pub p_IDM_pub p_ACC_alt_pub kp_range kd_range td_range ...
    a_range b_range T_range v0_range s0_range

p_ACC_pub       = [.23 .07 1.10];
p_IDM_pub       = [1.25 2.09 0.754 55 6.74];
p_ACC_alt_pub   = [.23 .07 0.840 6.38];

kp_range	= [.2 .3];             
kd_range 	= [.01 .1];
td_range   	= [1 3.6];

a_range   	= [1 3];
b_range 	= [1 3];
T_range   	= [1 3.6];
v0_range	= [45 55];
s0_range  	= [1 5];

%% Print calibration settings

fprintf(char(strcat( ...
    '\n ########################\n   Calibration summary \n\n', ...
    'Error measure \t\t\t %s \n ', ...
    'Calibration Variable \t %s \n ', ...    
    'Include gap error \t \t %d \nInclude speed error \t %d \n ', ...
    'Fraction gap \t\t\t %g \nFraction speed \t\t\t %g \n ', ...    
    ' ########################\n' ...
    )), ...
    calibration_setup.error_measure, ...
    calibration_setup.calibration_variable, ...
    calibration_setup.include_gap_err, ...
    calibration_setup.include_speed_err, ...
    calibration_setup.frac_gap, ...
    calibration_setup.frac_speed)

pause(2)

%% Select data
% Selected subsets of data
%       Subset      v_range     a_range     remarks
%       ---------   ---------   ---------   ---------
%       CF          Highway     [-1, 1]     car-following, low acc-/dec
%       OSC         Highway     [-2, 2]     oscillatory, moderate acc-/dec
%       SG1         Full        [-4, 2]     Full stop-and-go, high dec, moderate acc
%       SG2         Full        [-4, 4]     Full stop-and-go, high acc-/dec

% Remove invalid data / traffic lights
AUDI(AUDI.traffic_light == 1,:)     = [];
AUDI(AUDI.invalid == 1,:)           = [];
AUDI(~ismember(AUDI.regime_enum,[4,5]),:)	= [];

data_CF_noappend = AUDI(AUDI.Time > datetime([2018 06 13 10 17 00]) & ...
    AUDI.Time < datetime([2018 06 13 10 20 07]),:);
data_CF_append  = AUDI(AUDI.Time > datetime([2018 06 13 10 22 00]) & ...
    AUDI.Time < datetime([2018 06 13 10 24 00]),:);
data_CF         = [data_CF_noappend; data_CF_append];
data_CF         = sortrows(data_CF,'Time','ascend');

data_CI         = AUDI(AUDI.Time > datetime([2018 06 13 10 25 50]) & ...
    AUDI.Time < datetime([2018 06 13 10 30 51]),:);

data_CI2_noappend = AUDI(AUDI.Time > datetime([2018 06 13 14 36 30]) & ...
    AUDI.Time < datetime([2018 06 13 14 40 30]),:);
data_CI2_append = AUDI(AUDI.Time > datetime([2018 06 13 14 32 47]) & ...
    AUDI.Time < datetime([2018 06 13 14 33 51]),:);
data_CI2        = [data_CI2_noappend; data_CI2_append];
data_CI2        = sortrows(data_CI2,'Time','ascend');

data_HB         = AUDI(AUDI.Time > datetime([2018 06 13 14 40 50]) & ...
    AUDI.Time < datetime([2018 06 13 14 46 00]),:);

data_SG         = AUDI(AUDI.Time > datetime([2018 06 13 14 17 00]) & ...
    AUDI.Time < datetime([2018 06 13 14 22 04]),:);

% Add all data to struct
data_all        = add_calibration_info( ...
    data_CF, 'CF', ...
    data_CI, 'CI', ...
    data_CI2, 'CI2', ...
    data_HB, 'HB', ...
    data_SG, 'SG', ...
    data_CF_noappend, 'CF_short', ...
    data_CI2_noappend, 'CI2_short' ...
    );

FN              = fieldnames(data_all);

% Add benchmark trajectories
for i = 1 : numel(FN)
    % IDM model
    [~, data_all.(char(strcat(FN{i},'_IDM_gen'))), ~] = ...
        create_synthetic_data_IDM(data_all.(FN{i}).timetab, 1, false, p_IDM_pub);
    data_all.(char(strcat(FN{i},'_IDM_gen'))) = ...
        data_all.(char(strcat(FN{i},'_IDM_gen'))).traject1;
    
    % ACC model
    [~, data_all.(char(strcat(FN{i},'_ACC_gen'))), ~] = ...
        create_synthetic_data_ACC(data_all.(FN{i}).timetab, 1, false, p_ACC_pub);
    data_all.(char(strcat(FN{i},'_ACC_gen'))) = ...
        data_all.(char(strcat(FN{i},'_ACC_gen'))).traject1;
end
FN          = fieldnames(data_all);

%% Assess excitation of the trajectories

assess_excitation(data_all)

%% Create fitness landscape of the dataset
% plot_fitness_landscape_XXX(follower_data, plot_trajectory, ...
%   landscape_resolution)

close all
for i = 1 : 5
    plot_fitness_landscape_IDM(AUDI, data_all, FN{i}, N_points);
    plot_fitness_landscape_ACC(AUDI, data_all, FN{i}, N_points);
    plot_fitness_landscape_ACC_alt(AUDI, data_all, FN{i}, N_points);
end

%% Calibrate models

[p_IDM,cost_IDM,~,~]            = calibrate_IDM(data_all);
[p_ACC,cost_ACC,~,~]            = calibrate_ACC(data_all);
[p_ACC_alt,cost_ACC_alt,~,~]    = calibrate_ACC_alt(data_all);

outerjoin(table(FN,round(p_IDM,3,'Significant'),'VariableNames',{'Trajectory','p_IDM'}), ...
    outerjoin(table(FN,round(p_ACC,3,'Significant'),'VariableNames',{'Trajectory','p_ACC'}), ...
    table(FN,round(p_ACC_alt,3,'Significant'),'VariableNames',{'Trajectory','p_ACC_alt'}), ...
    'MergeKeys',true), ...
    'MergeKeys',true)

%% Validate models

% ( <data>, <option_label>, <option_value> )
[validation_table_IDM_RMSE, ~]      = cross_validate_IDM(data_all, ...
    'error_measure','RMSE','calibration_variable','sv');
[validation_table_ACC_RMSE, ~]      = cross_validate_ACC(data_all, ...
    'error_measure','RMSE','calibration_variable','sv');
[validation_table_ACC_alt_RMSE, p_ACC_vali_alt_RMSE] = cross_validate_ACC_alt(data_all, ...
    'error_measure','RMSE','calibration_variable','sv');

[validation_table_IDM_RMSREs, ~]    = cross_validate_IDM(data_all, ...
    'error_measure','RMSRE','calibration_variable','s');
[validation_table_ACC_RMSREs, ~]    = cross_validate_ACC(data_all, ...
    'error_measure','RMSRE','calibration_variable','s');
[validation_table_ACC_alt_RMSREs, p_ACC_vali_alt_RMSREs] = cross_validate_ACC_alt(data_all, ...
    'error_measure','RMSRE','calibration_variable','s');

[validation_table_IDM_RMSREv, ~]    = cross_validate_IDM(data_all, ...
    'error_measure','RMSRE','calibration_variable','v');
[validation_table_ACC_RMSREv, ~]    = cross_validate_ACC(data_all, ...
    'error_measure','RMSRE','calibration_variable','v');
[validation_table_ACC_alt_RMSREv, p_ACC_vali_alt_RMSREv] = cross_validate_ACC_alt(data_all, ...
    'error_measure','RMSRE','calibration_variable','v');

% Write all validation results to table_out
table_out = table();
for i = 1 : numel(FN)
    table_out   = [table_out; validation_table_IDM_RMSE(i,:); ...
        validation_table_IDM_RMSREs(i,:); ...
        validation_table_IDM_RMSREv(i,:); ...
        validation_table_ACC_RMSE(i,:); ...
        validation_table_ACC_RMSREs(i,:); ...
        validation_table_ACC_RMSREv(i,:); ...
        validation_table_ACC_alt_RMSE(i,:); ...
        validation_table_ACC_alt_RMSREs(i,:); ...
        validation_table_ACC_alt_RMSREv(i,:)]
end

%% Cross-compare model performance
% CF, CI, CI2, HB, SG
plot_traject    = 'SG';
idx_traject     = find(strcmpi(FN,plot_traject));

tmp_p_IDM_pub           = p_IDM_pub;
tmp_p_IDM_pub([3,5])    = mean(p_IDM(3:5,[3,5]));
tmp_p_ACC_pub           = p_ACC_pub;
tmp_p_ACC_pub(3)        = mean(p_ACC(3:5,3));
tmp_p_ACC_alt_pub       = tmp_p_ACC_pub;
tmp_p_ACC_alt_pub([3,4])= mean(p_ACC_alt(3:5,[3,4]));

% Select the desired validation error measure and calibration variable
tmp_orig_cali_setup = calibration_setup;
% err_meas_vec        = {'RMSRE','RMSRE'};
% cal_var_vec         = {'s','v'};
err_meas_vec        = {'RMSE'};
cal_var_vec         = {'sv'};


% plot_model_response(data, model1, p1, label1, model2, p2, label2 etc.)
close all
err_IDM = []; err_ACC = []; err_ACC_alt = []; 
progressbar(0,0)
for i = 1 : 2
    idx_traject     = i;
    plot_traject    = FN{i};
    for j = 1 : numel(err_meas_vec)
        progressbar((i-1)/5,(j-1)/numel(err_meas_vec))
        calibration_setup.error_measure         = err_meas_vec{j};
        calibration_setup.calibration_variable  = cal_var_vec{j};

        % Plot model response of desired model - parameter value pairs
        %   Three possible combinations are shown here.
        [err_IDM(i,j)]    = plot_model_response(AUDI,data_all.(plot_traject).timetab ...
            ... ,'IDM', p_IDM_pub, 'IDM (pub)' ...
            ... ,'IDM', tmp_p_IDM_pub, 'IDM (pub)' ...
             ,'IDM', mean(p_IDM(3:5,:)), 'IDM (mean)' ...
             ,'IDM', p_IDM(idx_traject,:), 'IDM' ...
            ... ,'IDM', p_IDM(3,:), 'IDM (cali)' ...
            ... ,'IDM', tmp_p_IDM(1,:), 'IDM (cali)' ...
             );

        [err_ACC(i,j)]    = plot_model_response(AUDI,data_all.(plot_traject).timetab ...
            ... ,'ACC', p_ACC_pub, 'ACC (pub)' ...
            ... ,'ACC', mean(p_ACC(3:5,:)), 'ACC (mean)' ...
            ... ,'ACC', tmp_p_ACC_pub, 'ACC (pub)' ...
             ,'ACC', p_ACC(idx_traject,:), 'ACC' ...
            ... ,'ACC', p_ACC(3,:), 'ACC (cali)' ...
            ... ,'IDM', p_IDM(idx_traject,:), 'IDM' ...
            );


        [err_ACC_alt(i,j)]  = plot_model_response(AUDI,data_all.(plot_traject).timetab ...
            ... ,'ACC_alternative', tmp_p_ACC_alt_pub, 'ACC alt (pub)' ...
            ... ,'ACC_alternative', mean(p_ACC_alt(3:5,:)), 'ACC alt (mean)' ...
            ... ,'ACC_alternative', p_ACC_alt(idx_traject,:), 'ACC alt (cali)' ...
             ,'ACC_alternative', p_ACC_alt(3,:), 'ACC alt (cali)' ...
            ... ,'ACC_alternative', mean(p_ACC_alt(3:5,:)), 'ACC alt (mean)' ...
            );
        

        %%%%%%%%%%%%%%%%%%% Error plots %%%%%%%%%%%%%%%%%%%%
        plot_model_response_debug(data_all.(plot_traject).timetab ...
             ,'IDM', mean(p_IDM(3:5,:)), 'IDM (mean)'...
             ,'IDM', p_IDM(idx_traject,:), 'IDM (cali)'...
            );
        
        plot_model_response_debug(data_all.(plot_traject).timetab ...
            ... ,'IDM', p_IDM(idx_traject,:), 'IDM' ...
            ... ,'IDM', mean(p_IDM(3:5,:)), 'IDM (mean)' ...
             ,'ACC', p_ACC(idx_traject,:), 'ACC (cali)' ...
             ,'ACC', mean(p_ACC(3:5,:)), 'ACC (mean)' ...
             ,'ACC_alternative', p_ACC_alt(idx_traject,:), 'ACC alt (cali)' ...
             ,'ACC_alternative', mean(p_ACC_alt(3:5,:)), 'ACC alt (mean)' ...
            );
        
        close([figure(2),figure(3)])
    end
end
progressbar(1,1)

% Print cross-comparision table (error one parameter pair on representing
% other trajectories)
close all
err_IDM     = reshape(err_IDM.',[1,numel(err_IDM)]);
fprintf('%g & %g & ', round(err_IDM,3,'significant'))
fprintf('\n')

err_ACC     = reshape(err_ACC.',[1,numel(err_ACC)]);
fprintf('%g & %g & ', round(err_ACC,3,'significant'))
fprintf('\n')

err_ACC_alt = reshape(err_ACC_alt.',[1,numel(err_ACC_alt)]);
fprintf('%g & %g & ', round(err_ACC_alt,3,'significant'))
fprintf('\n')

% Reset original calibration setup
calibration_setup   = tmp_orig_cali_setup;














